/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractAwaySelfCall.java
 * 
 * Time-stamp: <2003-03-07 23:44:47 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: AbstractAwaySelfCall.java,v 1.4 2003/10/13 07:25:42 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.* ;

/**
 * abstract awy calls from a class on itself.
 * @author Francois Taiani   <taiani@laas.fr>
 */
public class AbstractAwaySelfCall extends AbstractApplyToNodesCmd {

    public AbstractAwaySelfCall() {
 	super("absSelf",2,2) ;
	addAlias("sab");
	addAlias("remSelf");
	addAlias("$");
// 	setShortOutput(true) ;
	this.setShortDescription
	    ( "abstracts away internal calls for classes matching a pattern") ;
	this.setLongDescription
	    ( "'selfAbstract <aPattern> <graph>' abstracts away the nodes that "+
	      "represent a call from a class on itself. <aPattern> represents a "+
	      "a pattern of the classes this action should be applied on. " +
	      "A call from a class on itself has the same 'entity' attribute as its parent." +
	      "This is specific to call graphs and won't work on other graphs.") ;
   } // EndConstructor AbstractAwaySelfCall

    protected Collection getNodeSToRemoveFromCondition(String condition) 
    {
	if (!condition.endsWith("*"))
	    condition = condition+"::*" ;
	return graphUnderFire.getNodeSWithRegExp(condition);
    } // EndMethod getNodeSToRemoveFromCondition

    protected void applyCommandToNode(NamedVertex currentNamedVertex) 
	throws Exception
    {
	CallGraph callGraphUnderFire = (CallGraph)graphUnderFire ;
	if (callGraphUnderFire.eventIsASelfCall(currentNamedVertex)) {
	    _nbOfRemovedNodes++ ;
	    callGraphUnderFire.abstractAway(currentNamedVertex);
	} // EndIf callGraphUnderFire.eventIsASelfCall
    } // EndMethod applyCommandToNode

} // EndClass AbstractAwaySelfCall
