/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractLeapAndRebind.java.java
 * 
 * Time-stamp: <2003-03-11 13:49:50 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: AbstractLeapAndRebind.java,v 1.1 2008/10/30 14:32:19 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.* ;

/**
 * Abstract method for the leapover and rebind operations
 *
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision $
 */
public abstract class AbstractLeapAndRebind extends CommandWithVariableArgs{

    public AbstractLeapAndRebind(String aName) {
	super(aName,1,1) ;
   } // EndConstructor CommandWithVariableArgs

    public AbstractLeapAndRebind(String aName,int max,int min) {
	super(aName,max,min) ;
   } // EndConstructor CommandWithVariableArgs

    public abstract void doTransform(CallGraph sourceGraph, 
				     Set       startNodes, 
				     Set       endNodes) 
	throws Exception ;

    public boolean handleCall(String[] args) throws Exception {

       	if ( ! doNecessaryVariableChecks(args,3,3) ) return false ;
	boolean executionOk = false ;
	try {
	    CallGraph sourceGraph =
		(CallGraph)GraphVariable.GetGraphForName(args[3]) ;
	    Set startNodes = sourceGraph.getNodeSWithRegExp(args[1]) ;
	    Set endNodes   = sourceGraph.getNodeSWithRegExp(args[2]) ;

	    if ( startNodes.isEmpty() ) {
		println("No Matching Start Nodes in Graph '"+args[3]+"'.") ;
		return false ;
	    } 

	    if ( endNodes.isEmpty() ) {
		println("No Matching End Nodes in Graph '"+args[3]+"'.") ;
		return false ;
	    } 

	    doTransform(sourceGraph, startNodes, endNodes ) ;

	    println(getName() + " from "     + endNodes   .size() +
		    " end nodes to "  + startNodes .size() +
		    " start nodes from "+args[3]) ;

	    GraphVariable.NotifyChangeOnVariable(args[3],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[3]},
		  new String[] {args[3]}
		  );
	    executionOk = true ;
	} 
	catch (IllegalArgumentException except) {
	    println( "ERROR : " + except ) ;
	} // EndCatch

	return executionOk ;

    } // EndMethod handleCall

} // EndClass AbstractLeapAndRebind.java
