/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractPutCommand.java
 * 
 * Time-stamp: <2003-03-11 11:22:34 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AbstractPutCommand.java,v 1.3 2003/06/13 16:38:22 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDictImpl ;
import laas.friends.cosmopen.graphAnalysis.NamedVertex ;
import java.util.Collection ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public abstract class AbstractPutCommand extends CommandWithOptionalContext {

    public AbstractPutCommand(String aName) {
	super(aName,2,3,2) ;
    } // EndConstructor AbstractPutCommand

    /**
     * This methods computes the set of vertices that are added
     * from source variable (second or base) to the target graph. The 
     * extracting condition is expressed in the first parameter. It is abstract
     * in {@link AbstractRemove} and should be overwritten by
     * subclasses.
     */
    abstract protected Collection getNodeSToAddFromCondition
	( String condition, DirectedGraphWithDict sourceGraph) ;

    public boolean handleCall(String[] args) throws Exception {
	if ( ! doNecessaryVariableChecks(args,2,args.length-2)  ) return false;
	if ( CheckIsBaseGraphAndErrorIfYes(args[args.length-1]) ) return false ;

	boolean executionOk = false ;
	try {
	    String nameOfSource        = super.getContextName(args) ;
	    String nameOfGraphModified = args[args.length-1] ;

	    DirectedGraphWithDict sourceGraph 
		= GraphVariable.ProvideGraphValueForName (nameOfSource) ; 
	    DirectedGraphWithDict graphToBeModified 
		= GraphVariable.ProvideGraphValueForName (nameOfGraphModified) ;
	    java.util.Collection nodesToAdd = 
		getNodeSToAddFromCondition( args[1], sourceGraph );

	    if (nodesToAdd.isEmpty()) {
		println("No Matching Nodes in graph '"+nameOfSource+"'.") ;
		return false ;
	    } // EndIf nodesToAdd.isEmpty()

	    DirectedGraphWithDict newSubGraph =
		GraphVariable.NewEmptyGraphSameTypeAsBase() ;

	    newSubGraph.addNamedVertexS           ( nodesToAdd  ) ;
	    newSubGraph.doEdgeCompletionInContext ( sourceGraph ) ;

	    graphToBeModified.addGraph(newSubGraph) ;

	    // Printing the nodes which have been added.
	    
	    NamedVertex.PrintCollectionOfNamedVertex(nodesToAdd) ;
	    println("   -> "+nameOfGraphModified) ;
	    GraphVariable.NotifyChangeOnVariable(nameOfGraphModified,args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {nameOfGraphModified,nameOfSource},
		  new String[] {nameOfGraphModified}
		  );
	    
	    executionOk = true ;
	} 
	catch (IllegalArgumentException except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndClass AbstractPutCommand
