/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AddCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:57 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AddCommand.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.3
 */
public class AddCommand extends CommandWithVariableArgs {

    public AddCommand() {
	super("add",2,2) ;
	addAlias("a");
	addAlias("*");
	this.setShortDescription
	    ( "adds a graph to another.") ;
	this.setLongDescription
	    ( "'add <var1> <var2>' adds the edges and vertices of "+
	      "the graph contained in the variable <var1> to the one in the "+
	      "variable <var2>. If the <var2> has not yet been defined, it is "+
	      "instanciated.") ;
    } // EndConstructor AddCommand

    public boolean handleCall(String[] args) throws Exception {

	if ( ! doNecessaryVariableChecks(args,1,1)  ) return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(args[2]) ) return false ;
	boolean executionOk = false ;

	try {
	    DirectedGraphWithDict sourceGraph 
		= GraphVariable.GetGraphForName          (args[1]) ;
	    DirectedGraphWithDict graphToBeModified 
		= GraphVariable.ProvideGraphValueForName (args[2]) ;
	    graphToBeModified.addGraph(sourceGraph) ;
	    GraphVariable.NotifyChangeOnVariable(args[2],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[1],args[2]},
		  new String[] {args[2]}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch

	return executionOk ;

    } // EndMethod handleCall

} // EndClass AddCommand
