/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AssignCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:54 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AssignCommand.java,v 1.3 2004/10/27 17:16:52 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public class AssignCommand extends CommandWithVariableArgs {

    public AssignCommand() {
	super("assign",2,-1) ;
	addAlias("=>");
	this.setShortDescription
	    ( "assigns the graph of a given variable to a second.") ;
	this.setLongDescription
	    ( "'assign <var1> <var2> [<var3>..<varN>]' assigns the value "+
	      "of <var1> to <var2>, [<var3>..<varN>. "+
	      "You can use the alias 'base' to refer to the "+
	      "base graph. If <var2> does not exist it is created.") ;
    } // EndConstructor AssignCommand

    public boolean handleCall(String[] args) throws Exception {

	if ( ! doNecessaryVariableChecks(args,1,1) ) return false ;

	boolean executionOk = false ;

	DirectedGraphWithDict sourceGraph 
	    = GraphVariable.GetGraphForName (args[1]) ;
	java.util.List modifiedVar = new java.util.Vector() ;

	for (int i = 2; i<args.length; i++) {
	    GraphVariable
		.SetVariableToValue
		(args[i] , (DirectedGraphWithDict)sourceGraph.clone()) ;
	    if ( !CheckIsBaseGraphAndErrorIfYes(args[i]) ) {
		println("'"+args[1]+"' assigned to '"+args[i]+"'.") ;
		GraphVariable.NotifyChangeOnVariable(args[i],args) ;
		modifiedVar.add(args[i]) ;
	    } // EndIf !CheckIsBaseGraphAndErrorIfYes
	} // EndFor i

	GraphVariable.PutHistoryArgsInputOutput
	    ( args, 
	      new String[] {args[1]},
	      (String[])modifiedVar.toArray(new String[]{})
	      );
	executionOk = true ;
	return executionOk ;
    } // EndMethod handleCall

} // EndClass AssignCommand
