/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * BackNCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:50 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: BackNCommand.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public class BackNCommand extends AbstractBackForwCommand {

    public BackNCommand() {
	super("backN",2,3,BACKWARD) ;
	addAlias("bn");
	this.setShortDescription
	    ( "computes the backward set of a graph within a given depth.") ;
	this.setLongDescription
	    ( "'backN <depth> <seed> [<context>]' computes the backward "+
	      "set of the graph <seed> until a depth of <depth>."+
	      "The result of the operation is stored in the passed "+
	      "variable.") ;
    } // EndConstructor BackNCommand

    public boolean handleCall(String[] args) throws Exception {

	if ( ! doNecessaryVariableChecks(args,2,3) ) return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(args[2])) return false ;
	int depth = 0;
	try {
	    depth = Integer.parseInt(args[1]) ;
	    if (depth < 0) throw new NumberFormatException() ;
	} catch (NumberFormatException e) {
	    println("ERROR: "+args[1]+" is not a positive number.") ;
	    return false ;
	} ;

	return this.handleCallInternal (args,depth) ;

    } // EndMethod handleCall
} // EndClass BackNCommand
