/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * CommandWithOptionalContext.java
 * 
 * Time-stamp: <02/08/14 19:22:57 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: CommandWithOptionalContext.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;


/**
 * Abstract class for command with a last
 * optional argument ('context') which is replaced
 * by the base graphe (see {@link GraphVariable }) if
 * not present.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public abstract class CommandWithOptionalContext extends CommandWithVariableArgs {

    private int _positionOfContext = -1 ; // means last position

    public CommandWithOptionalContext(String commandName) {
	super(commandName,1,2) ;
    } // EndConstructor CommandWithOptionalContext

    public CommandWithOptionalContext(String commandName,
				      int    minArgs,
				      int    maxArgs ) {
	super(commandName,minArgs,maxArgs) ;
    } // EndConstructor CommandWithOptionalContext

    public CommandWithOptionalContext(String commandName,
				      int    minArgs,
				      int    maxArgs,
				      int    positionOfContext ) {
	super(commandName,minArgs,maxArgs) ;
	_positionOfContext = positionOfContext ;
    } // EndConstructor CommandWithOptionalContext

    /**
     * If args[2] exists, it is returned, otherwise
     * the name of the base graph is returned.
     */
    protected String getContextName(String[] args) {
	if (args.length==this.getMaxNbArgs()+1) { 
	    // maximum number of argument the command context is provided
	    // on the command line.
	    int actualPositionOfContext =  _positionOfContext ;
	    if (actualPositionOfContext<0) 
		actualPositionOfContext = this.getMaxNbArgs() ;
	    return args[actualPositionOfContext] ;
	} else {
	    return GraphVariable.NAME_OF_BASEALIAS ;
	} // EndElse only 1 args
    } // EndMethod getContextName
    
} // EndClass CommandWithOptionalContext
