/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * CommandWithVariableArgs.java
 * 
 * Time-stamp: <2003-03-11 11:27:52 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: CommandWithVariableArgs.java,v 1.3 2003/06/13 16:38:22 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;
import laas.friends.cosmopen.interpreter.* ;

/**
 * This class si the base graph for the commands of the 
 * OPSBrowser interpreter which works on graph
 * variables. It offers utilities to check the existence of the
 * variable args.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public abstract class CommandWithVariableArgs extends Command {

    public CommandWithVariableArgs(String aName) {
	super(aName,1,1) ;
   } // EndConstructor CommandWithVariableArgs

    public CommandWithVariableArgs(String aName,int max,int min) {
	super(aName,max,min) ;
   } // EndConstructor CommandWithVariableArgs

    /**
     * The same as 
     * {@link #doNecessaryVariableChecks(String[], int, int) doNecessaryVariableChecks} 
     * but when only the first args is to be checkes as a variable.
     */
    protected boolean passedVariableDoesExist(String[] args) {
	return doNecessaryVariableChecks(args,1,1);
    } // EndMethod passedVariableDoesExist

    /**
     * This methods is supposed to be called at the
     * beginning of {@link laas.friends.cosmopen.interpreter.Command#handleCall(String[]) 
     * handleCall}. It checks that there is only the right number of arguments
     * on the command line, and that the args in the range
     * <code>[rangeMin..rangeMax]</code> correspond to existing variables.
     * It also checks that a context base graph has been loaded.
     * If all that is true, it return true. Otherwise it prints an
     * appropriate error message and return false.
     */
    protected boolean doNecessaryVariableChecks
	(String[] args, int rangeMin, int rangeMax)
    {
	if ( ! thereIsRightNumberOfArgs(args) ) return false ;

	boolean isOk = true ; 
	// Tests if everything goes ok in the
	// following loop

	for(int i=rangeMin; (i<=rangeMax) && (i<args.length); i++) {
	    if ( ! GraphVariable.VariableWithNameIsKnown(args[i]) ) {
		println
		    ( "Variable with name '"+args[i]+"' is not defined.") ; 
		isOk = false ;
	    } // EndIf VariableWithNameIsKnown
	} // EndFor i

	if (!isOk) {
	    println
		( "Type 'help "+this.getName()+"' for more information.") ;
	    return false ;
	} // EndIf not isOK

	if (!GraphVariable.BaseGraphIsSet()) {
	    println
		( "No Base Graph is currently loaded. " +
		  "Type 'help clearall' for more information."
		  ) ; // EndCall println
	    return false ;
	} // EndIf there is no BaseGraph defined in GraphVariable.

	return true ;

    } // EndMethod doNecessaryVariableChecks

    static boolean CheckIsBaseGraphAndErrorIfYes(String aVarName) {
	boolean result = aVarName.equals(GraphVariable.NAME_OF_BASEALIAS) ;
	if (result) 
	    println
		( "ERROR: Trying to modify the base graph." ) ;
	return result ;
    } // EndMethod  IsBaseGraph

} // EndClass CommandWithVariableArgs
