/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * LoadCommand.java
 * 
 * Time-stamp: <02/08/12 18:32:09 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: LoadCommand.java,v 1.8 2005/10/27 12:18:13 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.interpreter.* ;
import laas.friends.cosmopen.graphAnalysis.* ;

public class LoadCommand extends Command {

    private final StopWatch myWatch = new StopWatch() ;

    public LoadCommand() {
	super("load",1,2) ;
	this.setShortDescription("loads a usage graph in Hydrogen format.") ;
	this.setLongDescription
	    ( "'load <aFileName> [<aVar>]' where '<aFileName>' is a file "+
	      "containing a graph in Hydrogen format (XML), "+
	      "loads the graph stored in the file into the OPSBrowser "+
	      "interpreter. \n"+
	      "The graph that is read from the file is added to a "+
	      "default variable called 'base'. This variable represents the "+
	      "base graph, and is a kind of reference graph for all further "+
	      "processing."+
	      "If a second argument is present, the graph is added to the "+
	      "variable with that name as well. If this variable did not "+
	      "exist before, it is instanciated.") ; 

	// [CHANGE] ftaiani Mon Jul  2 2001 08:48:23
	// Now loaded graphs are added to the passed variable, resp.
	// to the base graph.

// 	      "There can only be one base graph loaded at one point "+
// 	      "in time. An already loaded base graph must be explicitely "+
// 	      "unloaded with 'clearall' before any new base graph may be loaded. "+
// 	      "Type 'help clearall' for more information.") ;

    } // EndConstructor LoadCommand

    public boolean handleCall(String[] args) {
	if ( ! thereIsRightNumberOfArgs(args) ) return false ;
	boolean thereIsSecondArgs = args.length > 2 ;

	int count = 0 ;

	// [CHANGE] ftaiani Mon Jul  2 2001 08:48:23
	// Now loaded graphs are added to the passed variable, resp.
	// to the base graph.

// 	if (GraphVariable.BaseGraphIsSet() && !thereIsSecondArgs) {
// 	    println
// 		( "A Base Graph has already been loaded. You can't load " +
// 		  "a new Base Graph without calling 'clearall' before. " +
// 		  "Type 'help load' for more information."
// 		  ) ; // EndCall println
// 	    return false ;
// 	} // EndIf there is already a BaseGraph defined in GraphVariable.

	boolean executionOk = false ;

	try {
	    myWatch.startWatchWithMessage("Loading time: ") ;
	    java.io.InputStream  input  = new java.io.FileInputStream (args[1]) ;

	    GenericXMLReader reader = new GenericXMLReader() ; 
	    reader.setProgressListener
		(new TextProgressListener(CommandInterpreter.out,40));

	    // If there is a second argument, we store the graph read from
	    // file in a variable with a corresponding name as well.

	    if (thereIsSecondArgs) {
// 		DirectedGraphWithDict baseGraph
// 		    = GraphVariable.ProvideGraphValueForName
// 		                  (GraphVariable.NAME_OF_BASEALIAS);
// 		DirectedGraphWithDict graph 
// 		    = GraphVariable.ProvideGraphValueForName( args[2] );

		reader.setContextCreationListener
		    (new MyListenerAboutClassCreation(GraphVariable.NAME_OF_BASEALIAS)) ;
		reader.setMainRecipientCreationListener
		    (new MyListenerAboutClassCreation(args[2])) ;


		DirectedGraphWithDict baseGraph
		    = GraphVariable.GetGraphForNameNoExp
		                  (GraphVariable.NAME_OF_BASEALIAS);
		DirectedGraphWithDict graph 
		    = GraphVariable.GetGraphForNameNoExp( args[2] );

		reader.readGraphFromXML( graph, baseGraph, input ) ;
		GraphVariable.NotifyChangeOnVariable ( args[2], args ) ;
		GraphVariable.PutHistoryArgsInputOutput
		    ( args, 
		      new String[] {"base",args[2]},
		      new String[] {"base",args[2]}
		      );
	    } // EndIf thereIsSecondArgs
	    else {
		DirectedGraphWithDict graph 
		    = GraphVariable.GetGraphForNameNoExp
		                  (GraphVariable.NAME_OF_BASEALIAS);
		graph = reader.readGraphFromXML( graph, null,  input ) ;
		GraphVariable.SetVariableToValue(GraphVariable.NAME_OF_BASEALIAS,graph) ;
		GraphVariable.PutHistoryArgsInputOutput
		    ( args, 
		      new String[] {"base"},
		      new String[] {"base"}
		      );
	    } // EndElse no second arg

	    GraphVariable.NotifyChangeOnVariable
		( GraphVariable.NAME_OF_BASEALIAS, args ) ;
	    executionOk = true ;
	    myWatch.stopWatchAndPrintDuration() ;
	} catch (java.io.FileNotFoundException e) {
	    println( "\nERROR WHILE READING: "+
				"File '" + args[1] + "' could not be found.") ;
	} catch (org.xml.sax.SAXParseException parsingException) {
	    println( "\nERROR WHILE PARSING FILE: "
                                  + parsingException.getMessage()      + "\n" +
		     " in "       + args[1]                            + 
		     " (line "    + parsingException.getLineNumber()   + "," +
		     " column "   + parsingException.getColumnNumber() + "," +
		     " entity '" + parsingException.getPublicId()      + "')" ) ;
	    parsingException.printStackTrace() ;
	} catch (Exception e) {
	    println( "\nINTERNAL ERROR: " + e ) ;
	    e.printStackTrace() ;
	} // EndCatch

	return executionOk ;
    } // EndMethod handleCall

} // EndClass LoadCommand

class MyListenerAboutClassCreation implements GraphRecipientCreationListener {
    private String _nameOfVariableToSetOnCreation ;
    public MyListenerAboutClassCreation(String nameOfVariableToSetOnCreation)
    {
	_nameOfVariableToSetOnCreation = nameOfVariableToSetOnCreation ;
    }
    public void aGraphRecipientWasCreated(DirectedGraphWithDict createdGraph) {
	GraphVariable.SetVariableToValue(_nameOfVariableToSetOnCreation,createdGraph) ;	
    }
} // EndClass MyListenerAboutClassCreation

/**
 * This class mesures time between two events in the program. It works
 * like a stopwatch. If you want to measures different duration
 * concurrently you need to instantiate this class the corresponding
 * number of times
 */
class StopWatch {

    static public String currentSeqName = null ;

    private long startTime = -1 ;
    private String currentMessage ;

    public final void startWatchWithMessage(String message) {
        startTime      = System.currentTimeMillis();
        currentMessage = message;
    }
    
    public final void stopWatchAndPrintDuration() {
        long time = System.currentTimeMillis() - startTime;
        CommandInterpreter.out.println(currentMessage + time/1000+"."+time%1000+"s");
    } 

} // EndClass StopWatch
