/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * PrintCommand.java
 * 
 * Time-stamp: <02/08/08 16:44:52 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: PrintCommand.java,v 1.3 2003/10/13 07:25:42 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.interpreter.Command ;
import laas.friends.cosmopen.graphAnalysis.* ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public class PrintCommand extends Command {

    public PrintCommand() {
	super("print",1,2) ;
	this.setShortDescription
	    ( "prints the nodes of a graph possibly using a pattern.") ;
	this.setLongDescription
	    ( "'print <aVariableName>' prints the nodes of "+
	      "the passed variable graph.\n"+
	      "'print <aPattern> <aVariableName>' prints the nodes of "+
	      "the passed variable graph whose names match the passed pattern. "+
	      "You can use the alias 'base' to remove nodes directly from "+
	      "the base graph.\n"+
	      "<aPattern> must be a limited wildcard expression. I.e. stars "+
	      "('*') are allowed either at the beginning or at the end of the "+
	      "pattern but not in the middle.") ;
    } // EndConstructor PrintCommand
	
    public boolean handleCall(String[] args) throws Exception {
	if ( ! thereIsRightNumberOfArgs(args) ) return false ;

	String regexp ;
	String nameOfGraph ;

	if (args.length==2) { 
	    // There are 1 argument (remember that arg[0] is the
	    // name of the command). All nodes are printed. The unique
	    // argument is the name of a variable.
	    regexp      = "*" ;
	    nameOfGraph = args[1] ;
	} else { // There are 2 arguments
	    // There are 2 arguments. The first argument is a
	    // wildcard expression, the second the name of a
	    // graph variable.
	    regexp      = args[1] ;
	    nameOfGraph = args[2] ;
	} // EndIfElse

	if ( ! GraphVariable.VariableWithNameIsKnown(nameOfGraph) ) {
	    println
		( "Variable with name '"+nameOfGraph+"' is not defined.\n" +
		  "Type 'help "+this.getName()+"' for more information."
		  ) ; // EndCall println
	    return false ;
	} // EndIf VariableWithNameIsKnown

	boolean executionOk = false ;

	try {
	    DirectedGraphWithDict observedGraph 
		= GraphVariable.GetGraphForName(nameOfGraph) ;

	    java.util.SortedSet result =
		observedGraph.getNodeSWithRegExp(regexp);

	    if (result.isEmpty())
		println("No Matching Nodes in graph '"+nameOfGraph+"'.") ;
	    else {

		java.util.Iterator iter = result.iterator() ;
		while (iter.hasNext()) {
		    println(((NamedVertex)iter.next()).getName()) ;
		} // EndWhile Loop through result
	    } // EndElse there are matching nodes in the graph variable.
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndInnerClass PrintCommand

