/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * SaveCommand.java
 * 
 * Time-stamp: <2002-11-26 16:27:13 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: SaveCommand.java,v 1.4 2004/09/09 15:31:00 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;
import laas.friends.cosmopen.graphAnalysis.XMLGraphWriter ;

import java.io.* ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.4 $
 * @since Version 0.0.4
 */
public class SaveCommand extends CommandWithVariableArgs {

    public SaveCommand() {
	super("save",2,2) ;
	addAlias("sv");
	this.setShortDescription
	    ( "save the graph contained in a variable to an xml file." ) ;
	this.setLongDescription
	    ( "'save <aVar> <aFile>' saves the graph contained " +
	      "in a variable to an xml file and stores the history "+
	      "of <aVar> in a file with the same name, but an \"his\" ending." ) ;
     } // EndConstructor SaveCommand

    public boolean handleCall(String[] args) throws Exception {
	if ( ! doNecessaryVariableChecks(args,1,1) ) return false ;

	DirectedGraphWithDict sourceGraph =
	    GraphVariable.GetGraphForName(args[1]) ;

	try {
	    // Writing the graph

	    String theFileName = args[2] ;
	    if (!theFileName.endsWith(".xml")) theFileName += ".xml" ;

	    println("Writing "+args[1]+" to "+theFileName) ;

	    OutputStream output 
		= new FileOutputStream(theFileName) ;
	    XMLGraphWriter
		writer = new XMLGraphWriter(output);
	    writer.writeGraph(sourceGraph) ;
	    output.close() ;

	    String theHistoryFile = theFileName + ".his" ;

	    // Writing the history

	    println("Writing history to "+theHistoryFile) ;

	    PrintStream historyOutput 
		= new PrintStream(new FileOutputStream(theHistoryFile)) ;
	    historyOutput.println("History for '"+args[1]+"':");

	    // 'save' invocation should now appear in the global history
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[1]},
		  new String[] {args[1]}
		  );

	    History historyToPrint 
		= GraphVariable.GetHistoryForVariable(args[1]) ;

// 	    historyToPrint.add("save "+args[1]+" "+theFileName) ;
	    historyToPrint.printYourselfToStream(historyOutput) ;
	    historyOutput.close() ;

	} catch (IOException ioExcept) {
	    println( "IO ERROR: " + ioExcept) ;
	}
	return true ;
    } // EndMethod handleCall

} // EndClass SaveCommand
