/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * SliceCommand.java
 * 
 * Time-stamp: <2003-02-11 09:20:50 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: SliceCommand.java,v 1.3 2003/06/13 16:38:22 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.Collection ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.1.1
 */
public class SliceCommand extends AbstractPutCommand {

    public SliceCommand() {
	super("slice") ;
	addAlias("sli");
	addAlias("//");
	this.setShortDescription
	    ( "puts nodes from one graph to another w.r.t. a time interval.") ;
	this.setLongDescription
	    ( "'slice <interval> [<sourceGraph>] <targetGraph>' retrieves the nodes "+
	      "of a call graph whose event ID is included in the passed time slice into "+
	      "a target graph.\n"+
	      "<interval> is of the form <startID>-<endID>. If an ID is dropped, "+
	      "(for instance \"10-\"), the corresponding end is considered infinite. "+
	      "Note that the ending ID *is* included in the selection. "+
	      "<sourceGraph> is the name of a graph variable. This variable must "+
	      "already have been declared and initialized. \n"+
	      "<sourceGraph> is the name of a graph variable. If this "+
	      "variable already exists, the found nodes are added to "+
	      "the already existing graph. Otherwise a new variable with "+
	      "this name is instantiated and receives the result of the name "+
	      "name matching.\n"+
	      "When <sourceGraph> is not provided the base graph is used instead.") ;

    } // EndConstructor SliceCommand

    private String startStr = null ;
    private String endStr   = null ;

    protected Collection getNodeSToAddFromCondition
	( String condition, DirectedGraphWithDict sourceGraph) 
    {
	if (!(sourceGraph instanceof CallGraph))
	    throw new IllegalArgumentException("The passed graph is not a call graph.") ;
	int indexOfDash = condition.indexOf('-') ;
	if (indexOfDash<0)
	    throw new IllegalArgumentException
		("The passed interval is not of the form start-end.") ;

	startStr = condition.substring(0,indexOfDash) ;
	endStr   = condition.substring(indexOfDash+1) ;
	int start = stringBoundToInt( startStr ) ;
	int end   = stringBoundToInt( endStr   ) ;

	// We want to include the end of the interval in the resulting graph
	if (end!=-1) end++ ;
	
	return 
	    ((CallGraph)sourceGraph).getEventSBetweenEventID ( start, end ) ;	

    } // EndMethod getNodeSToAddFromCondition

    private int stringBoundToInt(String bound) {
	if (bound.length()==0) return -1 ;
	return Integer.parseInt(bound) ;
    } // EndMethod stringBoundToInt

    public boolean handleCall(String[] args) throws Exception {
	startStr = null ;
	endStr   = null ;
	boolean execWasOk = super.handleCall(args) ;
	if (execWasOk) {
	    // We put the full name of the events in the history as comments

	    String nameOfSource        = super.getContextName(args) ;
	    String nameOfGraphModified = args[args.length-1] ;

	    int start = stringBoundToInt( startStr ) ;
	    int end   = stringBoundToInt( endStr   ) ;

	    GraphVariable.PutHistoryArgsInputOutput
		(new String[]{ "## "+ GraphVariable.GetMostRecentEventNameBeforeEventID(start) }, 
		 new String[]{nameOfGraphModified,nameOfSource},
		 new String[]{nameOfGraphModified}
		  );
	    GraphVariable.PutHistoryArgsInputOutput
		(new String[]{ "## "+ GraphVariable.GetMostRecentEventNameBeforeEventID(end) }, 
		 new String[]{nameOfGraphModified,nameOfSource},
		 new String[]{nameOfGraphModified}
		  );
	} // EndIf execWasOk
	return execWasOk ;
    } // EndMethod handleCall
} // EndClass SliceCommand
