/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestGraphVariable.java
 * 
 * Time-stamp: <2003-02-04 15:03:58 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestGraphVariable.java,v 1.3 2004/09/09 15:31:00 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.interpreter.Command ;
import java.util.Vector ;
import java.util.List ;

/** *******************************************************
 * Test {@link GraphVariable}
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public class TestGraphVariable
    extends junit.framework.TestCase 
{

    /* ********************************************************/
    public TestGraphVariable(String name) {
	super(name) ;
    } // EndConstructor TestGraphVariable

    /* ********************************************************/
    public void testSmallProperties() {

	assertTrue(!GraphVariable.BaseGraphIsSet());

	// Accessing a variable which does not exist
	// throws an exception
	try {
	    GraphVariable.GetGraphForName("someName") ;
	    fail() ;
	} catch (Exception e) {
	} // EndTry

	// Defining a variable while no base graph has been
	// set should raise an exception

	try {
	    DirectedGraphWithDict aGraph
		= GraphVariable.ProvideGraphValueForName("aName") ;
	    fail() ;
	} catch (Exception e) {
	} // EndTry

	DirectedGraphWithDict myGraph = GraphVariable.NewEmptyGraphSameTypeAsBase() ;
	GraphVariable.SetBaseGraph(myGraph) ;
	assertTrue(GraphVariable.BaseGraphIsSet());

	DirectedGraphWithDict aGraph
	    = GraphVariable.ProvideGraphValueForName("myVar") ;

	assertSame("1",aGraph,GraphVariable.GetGraphForName         ("myVar")) ;
	assertSame("2",aGraph,GraphVariable.ProvideGraphValueForName("myVar")) ;

	assertTrue( GraphVariable.VariableWithNameIsKnown("myVar")) ;
	assertTrue(!GraphVariable.VariableWithNameIsKnown("someName")) ;

	// After clear has been called, a new base graph may be set
	// and previously declared variables are lost.

	GraphVariable.ClearAllVariables() ;
	assertTrue(!GraphVariable.BaseGraphIsSet());
	GraphVariable.SetBaseGraph(new DirectedGraphWithDictImpl()) ;
	assertTrue( GraphVariable.BaseGraphIsSet());

	assertTrue(!GraphVariable.VariableWithNameIsKnown("myVar")) ;
	assertTrue(!GraphVariable.VariableWithNameIsKnown("someName")) ;

	GraphVariable.ClearAllVariables() ;
    } // EndMethod testSmallProperties

    /** *******************************************************
     * the base graph (the "global context") (see {@link GraphVariable })
     * of the interpreter can also be reached with the variable alias base.
     * This alias can't be created by ProvideGraphValueForName,
     * and only exists when a BaseGraph is set.
     */
    public void testBaseAlias() {
	GraphVariable.ClearAllVariables() ;

	try {
	    GraphVariable.GetGraphForName("base") ;
	    fail() ;
	} catch (Exception e) {
	} // EndTry

	DirectedGraphWithDict myGraph = GraphVariable.NewEmptyGraphSameTypeAsBase() ;
	GraphVariable.SetBaseGraph(myGraph) ;
	myGraph.add(new NamedVertex("dummy") ) ;

	assertSame( myGraph, GraphVariable.GetGraphForName("base")  ) ;
	GraphVariable.ClearAllVariables() ;

    } // EndMethod testBaseAlias

    /** ********************************************************/
    public void testGetHistoryForVariable() {

	GraphVariable.ClearAllVariables() ;
	DirectedGraphWithDict myGraph = GraphVariable.NewEmptyGraphSameTypeAsBase() ;
	GraphVariable.SetBaseGraph(myGraph) ;

	// A new variable has an empty history

	GraphVariable.ProvideGraphValueForName("var1") ;
	assertTrue(GraphVariable.GetHistoryForVariable("var1").isEmpty()) ;
	assertTrue(GraphVariable.GetGlobalHistory().isEmpty()) ;

	GraphVariable.PutHistoryArgsInputOutput
	    ( new String[]{"put","*","base","var1"},
	      new String[]{"base"},
	      new String[]{"var1" } ) ;

	List oldHistoVar1 
	    = new Vector(GraphVariable
			 .GetHistoryForVariable("var1")) ;
	assertTrue( oldHistoVar1.size()==1) ;
	assertTrue( oldHistoVar1.get(0).equals("put * base var1") ) ;
	assertEquals
	    ( GraphVariable.GetHistoryForVariable("var1"),
	      GraphVariable.GetGlobalHistory() ) ;

	GraphVariable.ProvideGraphValueForName("var2") ;

	GraphVariable.PutHistoryArgsInputOutput
	    ( new String[]{"put","*","var1","var2"},
	      new String[]{"var1"},
	      new String[]{"var2" } ) ;

	assertTrue(GraphVariable.GetHistoryForVariable("var2").size()==2) ;
	assertEquals
	    ( oldHistoVar1,
	      GraphVariable
	      .GetHistoryForVariable("var1") ) ;
	List oldHistoVar2
	    = new Vector(GraphVariable
			 .GetHistoryForVariable("var2")) ;
	assertTrue ( oldHistoVar2.get(0).equals("put * base var1") ) ;
	assertTrue ( oldHistoVar2.get(1).equals("put * var1 var2") ) ;
	
	GraphVariable.PutHistoryArgsInputOutput
	    ( new String[]{"assign","var2","var1","base"},
	      new String[]{"var2"},
	      new String[]{"var1","base" } ) ;

	List expected = new Vector() ;
	expected.add("put * base var1") ;
	expected.add("put * var1 var2") ;
	expected.add("assign var2 var1 base") ;

	assertEquals ( expected,
		       GraphVariable
		       .GetHistoryForVariable("base") ) ;
	List newHistoVar1
	    = new Vector(GraphVariable
			 .GetHistoryForVariable("var1")) ;

	assertEquals ( expected, newHistoVar1 ) ; 
	
	GraphVariable.PutHistoryArgsInputOutput
	    ( new String[]{"clear","var1"},
	      new String[]{},
	      new String[]{"var1"} ) ;
	expected = new Vector() ;
	expected.add("clear var1") ;
	assertEquals 
	    ( expected,
	      GraphVariable.GetHistoryForVariable("var1") ) ; 

	assertEquals
	    ( oldHistoVar2,
	      GraphVariable
	      .GetHistoryForVariable("var2") ) ;

	expected = new Vector() ;
	expected.add("put * base var1") ;
	expected.add("put * var1 var2") ;
	expected.add("assign var2 var1 base") ;
	expected.add("clear var1") ;

	assertEquals(expected, GraphVariable.GetGlobalHistory()) ;

    } // EndMethod testGetHistoryForVariable

} // EndClass TestGraphVariable
