/*
 * Copyright 2004 Francois Taiani
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestBreakpointTraceExtractor.java
 * 
 * Time-stamp: <2004-10-01 11:38:52 ftaiani>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: TestBreakpointTraceExtractor.java,v 1.3 2005/03/11 19:53:45 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.extractor.test ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.extractor.* ;
import java.util.* ;
import java.io.* ;

/**
 * A test clas for TestBreakpointTraceExtractor
 * @author Francois Taiani   <f.taiani@computer.org>
 */
public class TestBreakpointTraceExtractor extends junit.framework.TestCase {
    
    final private String testBreakpointTraceExtractor = 
        "laas.friends.cosmopen.extractor.test.TestBreakpointTraceExtractor" ;

    public TestBreakpointTraceExtractor(String name) {
	super(name) ;
    } // EndConstructor TestBreakpointTraceExtractor

    private GenericXMLReader graphReader = new GenericXMLReader();

    private CallGraph readGraphFromString ( String aGraphDescription  ) 
	throws Exception {

	return (CallGraph)graphReader.readGraphFromXML
	    ( new ByteArrayInputStream (aGraphDescription.getBytes()) ) ;

    } // EndMethod readGraphFromString
   

    private CallGraph constructCallGraphFromTracing(BreakpointTraceExtractor testee) 
        throws Exception 
    {
        
        ByteArrayOutputStream outResult = new ByteArrayOutputStream() ;
        testee.setOutputStream(outResult);
        testee.collectEventTillVMDeath();
        String resultingTrace = outResult.toString() ;

        GenericXMLReader graphReader = new GenericXMLReader();

        return readGraphFromString(resultingTrace) ;
        
    } // EndMethod constructCallGraphFromTracing

    private final String plainGuineaPig = 
        "laas.friends.cosmopen.extractor.test.PlainGuineaPig" ;

    public void testPlainTraceExtraction() throws Exception {

	System.err.println("testPlainTraceExtraction") ;
        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         plainGuineaPig, null) ;

        testee.setBreakPoint( plainGuineaPig, "method3a") ;
        testee.setBreakPoint( plainGuineaPig, "method3b") ;

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method3a\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method3b\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testPlainTraceExtraction

    public void testAdaptiveExtraction() throws Exception {

	System.err.println("testAdaptiveExtraction") ;

        String adaptiveGuineaPig = 
            "laas.friends.cosmopen.extractor.test.AdaptiveGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         adaptiveGuineaPig, null) ;

  
        ByteArrayOutputStream outResult = new ByteArrayOutputStream() ;

        testee.setOutputStream(outResult);
        testee.setBreakPoint( adaptiveGuineaPig, "method2") ;
        testee.collectEventTillNextBreakPoint() ; // Adaptation :-)
        testee.setBreakPoint( adaptiveGuineaPig, "method1") ;
        testee.collectEventTillVMDeath();

        String resultingTrace = outResult.toString() ;

        GenericXMLReader graphReader = new GenericXMLReader();

        CallGraph resultingGraph = readGraphFromString(resultingTrace) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"method2\" />" + 
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"method1\" />" + 
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testAdaptiveExtraction

    public void testAdaptiveExtraction2() throws Exception {

	System.err.println("testAdaptiveExtraction2");

        String adaptiveGuineaPig = 
            "laas.friends.cosmopen.extractor.test.AdaptiveGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         adaptiveGuineaPig, null) ;

  
        ByteArrayOutputStream outResult = new ByteArrayOutputStream() ;

        testee.setOutputStream(outResult);
        testee.setBreakPoint( adaptiveGuineaPig, "method2") ;
        testee.collectEventTillGivenBreakPoint(adaptiveGuineaPig, "method2") ; 
        // Adaptation :-)
        testee.setBreakPoint( adaptiveGuineaPig, "method1") ;
        testee.collectEventTillVMDeath();

        String resultingTrace = outResult.toString() ;

        GenericXMLReader graphReader = new GenericXMLReader();

        CallGraph resultingGraph = readGraphFromString(resultingTrace) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"method2\" />" + 
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"method1\" />" + 
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testAdaptiveExtraction2

    /**
     * Tests that {@link
     * BreakpointTraceExtractor#collectEventTillGivenBreakPoint(String,
     * String) collectEventTillGivenBreakPoint} continues collecting
     * events untill the death of the target VM when the passed
     * breakpoint does not exist.
     */
    public void testAdaptiveExtraction3() throws Exception {

	System.err.println("testAdaptiveExtraction3");

        String adaptiveGuineaPig = 
            "laas.friends.cosmopen.extractor.test.AdaptiveGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         adaptiveGuineaPig, null) ;

  
        ByteArrayOutputStream outResult = new ByteArrayOutputStream() ;

        testee.setOutputStream(outResult);
        testee.setBreakPoint( adaptiveGuineaPig, "method2") ;
        testee.collectEventTillGivenBreakPoint(adaptiveGuineaPig, "method1") ; 
        assertTrue(!testee.targetJVMisConnected()) ;

        String resultingTrace = outResult.toString() ;

        GenericXMLReader graphReader = new GenericXMLReader();

        CallGraph resultingGraph = readGraphFromString(resultingTrace) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"method2\" />" + 
	    "    <call entity=\""+adaptiveGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testAdaptiveExtraction3

    public void testPatternBasedTraceExtraction() throws Exception {

	System.err.println("testPatternBasedTraceExtraction");

        final String secondPlainGuineaPig 
            = "laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor + " " +
                                         secondPlainGuineaPig, null) ;
        
        testee.restrictTraceToPackage("laas.friends.cosmopen.extractor.test.") ;
        testee.setBreakPointExpression(secondPlainGuineaPig) ; 

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"&lt;init>\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method3a\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testPatternBasedTraceExtraction
    
    public void testPatternBasedTraceExtraction2() throws Exception {

	System.err.println("testPatternBasedTraceExtraction2") ;

        final String secondPlainGuineaPig 
            = "laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor + " " +
                                         secondPlainGuineaPig, null) ;
        
        testee.restrictTraceToPackage("laas.friends.cosmopen.extractor.test.") ;
        testee.setBreakPointExpression(secondPlainGuineaPig) ; 
        testee.setBreakPointExpression(plainGuineaPig) ;

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"&lt;init>\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"&lt;init>\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method4\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method3a\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method3b\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.SecondPlainGuineaPig\" "+
            "          method=\"method2\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"method1\" />" + 
	    "    <call entity=\"laas.friends.cosmopen.extractor.test.PlainGuineaPig\" "+
            "          method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" "+
            "          method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testPatternBasedTraceExtraction2


    public void testTraceExtractionWithThreads() throws Exception {
    
	System.err.println("testTraceExtractionWithThreads");

        final String threadedGuineaPig 
            = "laas.friends.cosmopen.extractor.test.ThreadedGuineaPig" ;
        final String threadThatIsCreated
            = "laas.friends.cosmopen.extractor.test.ThreadThatIsCreatedByGuineaPig" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor + " " +
                                         threadedGuineaPig, null) ;
        
        testee.setBreakPoint( threadThatIsCreated, "method3a") ;
        testee.setBreakPoint( "java.lang.Thread" , "start") ;

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        String expectedTrace =
	    "<traceSet threadCreation=\"java.lang.Thread::start\" >" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"java.lang.Thread\" method=\"start\" />" + 
	    "    <call entity=\""+threadedGuineaPig+"\" method=\"method2\" />" + 
	    "    <call entity=\""+threadedGuineaPig+"\" method=\"method1\" />" + 
	    "    <call entity=\""+threadedGuineaPig+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\""+threadThatIsCreated+"\" method=\"method3a\" />" + 
	    "    <call entity=\""+threadThatIsCreated+"\" method=\"run\" />" + 
            "  </trace> " +
	    "  <end thread=\"2\" />" +
	    "  <end thread=\"1\" />" +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testTraceExtractionWithThreads

    public void testOptionPassing() throws Exception {
    
	System.err.println("testOptionPassing");

        final String testingOptionPassing 
            = "laas.friends.cosmopen.extractor.test.TestingOptionPassingClass" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor + " " +
                                         testingOptionPassing, null) ;
        
        testee.setBreakPoint( testingOptionPassing, "method1" ) ;

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        CallGraph emptyGraph = new CallGraph() ;

        assertEquals(emptyGraph, resultingGraph ) ;

        testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor + " " +
                                         testingOptionPassing, 
                                         "-DsomePropertyName=foo") ;

        testee.setBreakPoint( testingOptionPassing, "method1") ;

        String expectedTrace =
	    "<traceSet threadCreation=\"java.lang.Thread::start\" >" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+testingOptionPassing+"\" method=\"method1\" />" + 
	    "    <call entity=\""+testingOptionPassing+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph  = readGraphFromString(expectedTrace) ;
        
        resultingGraph = constructCallGraphFromTracing(testee) ;

        assertEquals(expectedGraph, resultingGraph ) ;

        // The same as before, but with a different constructor

        testee = 
            new BreakpointTraceExtractor
            (new String[] {"-DsomePropertyName=foo",
                           testBreakpointTraceExtractor,
                           testingOptionPassing } ) ;

        testee.setBreakPoint( testingOptionPassing, "method1") ;

        resultingGraph = constructCallGraphFromTracing(testee) ;
        assertEquals(expectedGraph, resultingGraph ) ;

    } // EndMethod testOptionPassing

    public void testExceptionExtraction() throws Exception {

	System.err.println("testExceptionExtraction");

        final String anotherClass = 
            "laas.friends.cosmopen.extractor.test.AnotherClassForTestingTracing" ;

        BreakpointTraceExtractor testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         anotherClass, null) ;

        testee.setBreakPoint( anotherClass, "foo2") ;

        CallGraph resultingGraph = constructCallGraphFromTracing(testee) ;
                                          
        String expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+anotherClass+"\" method=\"foo2\" />" + 
	    "    <call entity=\""+anotherClass+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals("Exceptions desactivated",expectedGraph, resultingGraph ) ;

        // Second test : with Exception collection activated

        testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         anotherClass, null) ;

        testee.setBreakPoint( anotherClass, "foo2") ;
        testee.activateExceptionExtraction(true) ;

        resultingGraph = constructCallGraphFromTracing(testee) ;

        final String otherClass = 
            "laas.friends.cosmopen.extractor.test.OtherClass" ;
                                          
        expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"java.lang.RuntimeException\" method=\"throw\" />" + 
	    "    <call entity=\""+  otherClass+"\" method=\"foo\" />" + 
	    "    <call entity=\""+anotherClass+"\" method=\"foo\" />" + 
	    "    <call entity=\""+anotherClass+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+anotherClass+"\" method=\"foo2\" />" + 
	    "    <call entity=\""+anotherClass+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals("Exceptions activated",expectedGraph, resultingGraph ) ;

        // Third test : with Exception collection activated and
        // immediately desactivated

        testee = 
            new BreakpointTraceExtractor(testBreakpointTraceExtractor +" "+ 
                                         anotherClass, null) ;

        testee.setBreakPoint( anotherClass, "foo2") ;
        testee.activateExceptionExtraction(true) ;
        testee.activateExceptionExtraction(false) ;

        resultingGraph = constructCallGraphFromTracing(testee) ;

        expectedTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\""+anotherClass+"\" method=\"foo2\" />" + 
	    "    <call entity=\""+anotherClass+"\" method=\"myStartIsHere\" />" + 
	    "    <call entity=\""+testBreakpointTraceExtractor+"\" method=\"main\" />" +
            "  </trace> " +
	    "</traceSet>" ; 

        expectedGraph = readGraphFromString(expectedTrace) ;

        assertEquals("Exceptions activated/desactivated",expectedGraph, resultingGraph ) ;

    } // EndMethod testExceptionExtraction

    /**
     * This main function is used during the test to launch various
     * subclasses of AbstractGuineaPig. The goal is to test the
     * tracing capabilities of {@link BreakpointTraceExtractor}.
     */
    public static void main(String[] argv) throws Exception {

        System.err.println("TestBreakpointTraceExtractor.main");
        
         AbstractGuineaPig aGuineaPigClass 
                  = (AbstractGuineaPig)Class.forName(argv[0]).newInstance() ;
// 	AbstractGuineaPig aGuineaPigClass = new PlainGuineaPig();
//         aGuineaPigClass.myStartIsHere() ;

        System.err.println("Leaving TestBreakpointTraceExtractor.main");

    } // EndMethod main

} // EndClass TestBreakpointTraceExtractor

