/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * ClassDiagramRenderer.java
 * 
 * Time-stamp: <2002-08-16 22:19:02 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: ClassDiagramRenderer.java,v 1.4 2003/10/20 08:44:52 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * Renders the usage and inheritance links in dot.
 * @see PropertyRenderer
 * @see DirectedGraphWithDict2DotOStream
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.4 $
 * @since Version 0.1.0
 */
public class ClassDiagramRenderer 
    implements PropertyRenderer {

    public String formatProperty(String      kindOfLink,
				 PropertyBag propOfLink,
				 NamedEdge    linkToBeRendered,
				 DirectedGraphWithDict contextGraph
				 ) 
    {
	StringBuffer result = new StringBuffer();

	String nameOfLink =  propOfLink.getValueForProperty("name") ;
	String labelToBeWritten = "" ;

	if (nameOfLink!=null)
	    labelToBeWritten = labelToBeWritten + nameOfLink ;

	if (kindOfLink!=null) {

	    if (kindOfLink.equals("superclass")) {
		result.append("arrowhead=empty arrowsize=2 ") ;
		if (DirectedGraphWithDict2DotOStream.useShortEntityName)
		    labelToBeWritten ="" ;
	    }

	    if (kindOfLink.equals("usage")) {
		    
		result.append("style=solid ") ;

		// We assume (wrongly) that every template is a container, and
		// as such adds a reference level to the objects it contains
		// Consequently a usage link is indentifies as a composition 
		// only when no referencing ('*' and '&'
		// C++ operator) was found, *and* the type of the link is not
		// a template instance.

		int referenceDepth 
		    = Integer.parseInt
		    (propOfLink.getValueForProperty("referenceDepth"));

		boolean isTemplateInstance 
		    = (propOfLink.getValueForProperty("template")!=null) ;
		boolean isLinkFromPseudoClass
		    = ( ((NamedVertex)linkToBeRendered.getSource())
			.getPropBag()
			.getValueForProperty("pseudoClass") != null ) ;

		boolean isComposition =
		    (!isLinkFromPseudoClass) && 
		    (!isTemplateInstance)    && 
		    (referenceDepth==0) ;

		boolean isStatic           
		    = (propOfLink.getValueForProperty("isStatic")!=null) ;

		if (isComposition) {
		    result.append("arrowtail=diamond ") ;
		    //result.append("headlabel=\"1\" taillabel=\"1\" ") ;
		    result.append("headlabel=\"1\" labeldistance=2 ") ;
		} // EndIf is a composition

		else if (referenceDepth>1) {
		    result.append("headlabel=\"n\"") ;
		} // EndElseIf referenceDepth>0

		if (isStatic)
		    result.append("arrowtail=odot ") ;

		String  type = propOfLink.getValueForProperty("template") ;
		boolean isLinkToPseudoClass
		    = (propOfLink.getValueForProperty("isPseudo")!=null) ;

		if ((type!=null) && !isLinkToPseudoClass)
		    labelToBeWritten = 
			labelToBeWritten + "\\n" +
			DirectedGraphWithDict2DotOStream.breakDownString(type) ;

	    } // EndIf Kind is usage
	} // EndIf kind not null

	if (!labelToBeWritten.equals(""))
	    result.append(  "label=\"" + labelToBeWritten + "\" " ) ;

	return result.toString() ;

    } // EndMethod formatProperty

} // EndClass ClassDiagramRenderer
