/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * ProgressListener.java
 * 
 * Time-stamp: <01/04/11 14:43:30 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: ProgressListener.java,v 1.2 2003/04/07 18:10:36 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

/**
 * A {@link ProgressListener } is used to get feedback
 * on the progress of a given activity of a class which 
 * implements the {@link ProgressReporting } interface.
 * <P>
 * The normal way of feeding such a listener for a working class is
 * the following:
 * <PRE>
 *     aProgressListener.start("I'm starting an Activity.") ;
 *     ...
 *     aProgressListener.tick() ; // Each time a progress is made
 *     ...                        // here more ticks
 *     aProgressListener.end() ;
 * <PRE>
 * @see ProgressReporting
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.2
 */
public interface ProgressListener {

    /**
     * is called each time the working class has made
     * "some progress" in its activity. (Very loose notion.)
     * This method can for example put a dot on the screen.
     */
    public void tick() ;

    /**
     * is called by the working class when it starts 
     * a new activity. The passed message contains information
     * about that start.
     */
    public void start(String message) ;

    /**
     * is called by the working class when is stops
     * an activity.
     */
    public void end() ;

} // EndClass ProgressListener
