/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TextProgressListener.java
 * 
 * Time-stamp: <01/07/04 18:29:06 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TextProgressListener.java,v 1.2 2003/04/07 18:10:36 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

/**
 * This is a default implementation of a {@link ProgressListener }
 * which report progress of an actibity on a stream in a 
 * textual way.<P>
 * Typically:
 * <PRE>
 *    I'm doing something:
 *    .......
 * </PRE>
 * @see ProgressListener
 * @see ProgressReporting
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.2
 */
public class TextProgressListener 
    implements ProgressListener
{
    private java.io.PrintStream _theStreamIWriteTo ;
    private int   _samplingFreq = 1 ;
    private int   _count = 0 ;  
    
    public TextProgressListener(java.io.OutputStream anOStream) {
	_theStreamIWriteTo = new java.io.PrintStream(anOStream) ;
    } // EndConstructor TextProgressListener

    /**
     * @param samplingFreq 1 (default) is the highest and will produce
     *                     a fast ProgressListener
     */
    public TextProgressListener
	(java.io.OutputStream anOStream,
	 int                  samplingFreq ) {
	_theStreamIWriteTo = new java.io.PrintStream(anOStream) ;
	_samplingFreq      = samplingFreq ;
    } // EndConstructor TextProgressListener

    /**
     * Simply write a point '.' of the stream
     */
    public void tick() {
	_count++ ;
	if (_count % _samplingFreq ==0) {
	    _theStreamIWriteTo.print(".") ;
	    _count = 0 ;
	}
    } // EndMethod

    public void start(String message) {
	_theStreamIWriteTo.println(message) ;
    } // EndMethod

    public void end() {
	_theStreamIWriteTo.println() ;
    } // EndMethod

} // EndClass TextProgressListener
