/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractCallGraphReaderTest.java
 * 
 * Time-stamp: <2003-03-07 20:07:13 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AbstractCallGraphReaderTest.java,v 1.4 2004/09/09 15:31:01 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;

import java.io.ByteArrayInputStream    ;
import java.io.InputStream    ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.4 $
 * @since Version 0.1.1
 */
abstract public class AbstractCallGraphReaderTest 
    extends BagBasedTest {

    public AbstractCallGraphReaderTest(String name) {
	super(name) ;
    } // EndConstructor AbstractCallGraphReaderTest

    public static InputStream getTraceSetXMLStream1() {
	String XMLString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"D\" method=\"callOnD\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"callOnE\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
// 	    "  <trace> " +
// 	    "    <call entity=\"E\" method=\"callOnE\" />" +
// 	    "    <call entity=\"B\" method=\"callOnB\" />" +
// 	    "    <call entity=\"A\" method=\"callOnA\" />" + 
// 	    "  </trace> " +
// 	    "  <trace> " +
// 	    "    <call entity=\"C\" method=\"callOnC\" />" +
// 	    "    <call entity=\"B\" method=\"callOnB\" />" +
// 	    "    <call entity=\"A\" method=\"callOnA\" />" + 
// 	    "  </trace> " +
	    "</traceSet>" ; 
	return new ByteArrayInputStream(XMLString.getBytes()) ;

    } // EndMethod getTraceSetXMLString

    public static DirectedGraphWithDict getExpectedGraphForStream1() throws Exception {
	String expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::callOnA'0\" entity=\"A\" method=\"callOnA\" eventOrder=\"0\" > " +
	    "    <link target = \"B::callOnB'1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::callOnB'1\" entity=\"B\" method=\"callOnB\" eventOrder=\"1\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::callOnC'2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::callOnC'2\" entity=\"C\" method=\"callOnC\" eventOrder=\"2\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "  <class name=\"D::callOnD'3\" entity=\"D\" method=\"callOnD\" eventOrder=\"3\" > " +
	    "    <link target = \"B::callOnB'4\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::callOnB'4\" entity=\"B\" method=\"callOnB\" eventOrder=\"4\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::callOnC'5\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::callOnC'5\" entity=\"C\" method=\"callOnC\" eventOrder=\"5\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "  <class name=\"A::callOnA'6\" entity=\"A\" method=\"callOnA\" eventOrder=\"6\" > " +
	    "    <link target = \"B::callOnB'7\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::callOnB'7\" entity=\"B\" method=\"callOnB\" eventOrder=\"7\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"E::callOnE'8\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"E::callOnE'8\" entity=\"E\" method=\"callOnE\" eventOrder=\"8\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;

	XMLGraphReader graphReader = new XMLGraphReader();

	return
	    graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expectedResult.getBytes()) ) ;
	
    } // EndMethod getExpectedGraphForString1

} // EndClass AbstractCallGraphReaderTest
