/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestDirectedGraphWithDictImpl.java
 * 
 * Time-stamp: <2003-02-24 18:59:16 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestDirectedGraphWithDictImpl.java,v 1.5 2004/10/27 17:16:53 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import java.io.ByteArrayInputStream    ;

import laas.friends.cosmopen.utils.PropertyBag ;
import laas.friends.cosmopen.graphAnalysis.* ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.5 $
 * @since Version 0.0.2
 */
public class TestDirectedGraphWithDictImpl
extends BagBasedTest {

    // **************************************************************
    public TestDirectedGraphWithDictImpl(String name) {
	super(name) ;
    } // EndConstructor TestDirectedGraphWithDictImpl

    // **************************************************************
    public void testDirectedGraphWithDictImplBasics() 
	throws Exception 
    {

	DirectedGraphWithDict
	    graphWithDictTestee = new DirectedGraphWithDictImpl() ;
	NamedVertex premiere      = new NamedVertex("premiere") ;
	NamedVertex seconde       = new NamedVertex("seconde") ;
	NamedVertex faussePremiere= new NamedVertex("premiere") ;

	graphWithDictTestee.add(premiere) ;
	assertSame( premiere,
		    graphWithDictTestee
		    .getNodeWithName("premiere")
		    ); // EndCall assertSame

	assertNull( graphWithDictTestee
		    .getNodeWithName("seconde")
		    ); // EndCall assertNull

	graphWithDictTestee.add(seconde) ;

	assertSame( premiere,
		    graphWithDictTestee
		    .getNodeWithName("premiere")
		    ); // EndCall assertSame

	assertSame( seconde,
		    graphWithDictTestee
		    .getNodeWithName("seconde")
		    ); // EndCall assertSame

	// [CHANGE]: Not the case any more.
// 	// Adding a vertex with the same name should throw
// 	// an exception.

// 	try {
// 	    graphWithDictTestee.add(faussePremiere) ;
// 	    fail() ;
// 	} catch (Exception e) {
// 	}
	
	salvo.jesus.graph.Edge 
	    aLink = graphWithDictTestee.addEdge(premiere,seconde) ;
	assertSame( aLink,
		    graphWithDictTestee
		    .getIncomingEdges(seconde )
		    .iterator().next() 
		    ) ;

	assertTrue(graphWithDictTestee.getIncomingEdges(premiere).isEmpty()) ;

	// Links identity should be based on links names. Creating
	// 2 links between 2 nodes with the same name should result
	// in only one links

	NamedEdge linkData1 = new NamedEdge(premiere,seconde,"commonName") ;
	NamedEdge linkData2 = new NamedEdge(premiere,seconde,"commonName") ;

	graphWithDictTestee.addEdge(linkData1) ;

	int[][][] nbOf = new int[2][2][2];

	int incoming    = 0 ;
	int outgoing    = 1 ;
	int before      = 0 ;
	int after       = 1 ;
	NamedVertex[] nodes = new NamedVertex[] {premiere,seconde} ;

	// We remember the cardinalities of all nodes for both
	// the incoming and the outgoing direction.

	for(int nodeNb = 0; nodeNb <2 ; nodeNb++) {
	    nbOf[incoming][before][nodeNb]= 
		graphWithDictTestee
		.getIncomingEdges(nodes[nodeNb]).size() ;
	    nbOf[outgoing][before][nodeNb] =
		graphWithDictTestee
		.getOutgoingEdges(nodes[nodeNb]).size() ;
	} // EndFor nodeNb

	graphWithDictTestee.addEdge(linkData2) ;

	// We know compare the old cardinalities to the old ones. They
	// should be the same.

	for(int nodeNb = 0; nodeNb <2 ; nodeNb++) {
	
	    nbOf[incoming][after][nodeNb]= 
		graphWithDictTestee
		.getIncomingEdges(nodes[nodeNb]).size() ;
	    nbOf[outgoing][after][nodeNb] =
		graphWithDictTestee
		.getOutgoingEdges(nodes[nodeNb]).size() ;

	    for(int direction = 0; direction <2 ; direction++) {
		assertEquals( "direction: "+direction+", nodeNb: "+nodeNb,
			      nbOf[direction][before][nodeNb] ,
			      nbOf[direction][after] [nodeNb] ) ;
	    } // EndFor direction

	} // EndFor nodeNb

	// Insering a third link with the same name as the 2 previous ones,
	// but with different target should modify the graph. (Different link)

	NamedEdge linkData3 = new NamedEdge(premiere,premiere,"commonName") ;

	graphWithDictTestee.addEdge(linkData3) ;

	for(int nodeNb = 0; nodeNb <2 ; nodeNb++) {
	
	    nbOf[incoming][after][nodeNb]= 
		graphWithDictTestee
		.getIncomingEdges(nodes[nodeNb]).size() ;
	    nbOf[outgoing][after][nodeNb] =
		graphWithDictTestee
		.getOutgoingEdges(nodes[nodeNb]).size() ;

	} // EndFor nodeNb

	// Node premiere (0): Should have one more out, one more in.

	for(int direction = 0; direction <2 ; direction++) {
	    assertEquals( "ThirdLink : direction: "+direction+", nodeNb: "+0,
			  nbOf[direction][before][0]+1 ,
			  nbOf[direction][after] [0]  ) ;
	} // EndFor direction

	// Node seconde (1): Should remain the same.

	for(int direction = 0; direction <2 ; direction++) {
	    assertEquals( "ThirdLink : direction: "+direction+", nodeNb: "+1,
			  nbOf[direction][before][1] ,
			  nbOf[direction][after] [1] ) ;
	} // EndFor direction

    } // EndMethod testDirectedGraphWithDictImplBasics

    // [CHANGE] (Francois Taiani) Wed Apr 17 2002 14:23:20
    private GraphTesteeCreator 
	testeeCreator = new GraphTesteeCreator() ;

    private static String graph2String =
	    "<project>" +

	    "  <class name=\"Arbre\">"           +
	    "    <link target = \"E\" />" +
	    "  </class> " +

	    "  <class name=\"E\">"           +
	    "  </class> " +

	    "</project>" ;


    /****************************************************************
     * create an exemplary graph to be used in some test methods.
     */
    private DirectedGraphWithDict createGraphTestee() 
	throws Exception 
    {
	return testeeCreator.createGraphTestee() ;
    } // EndMethod createGraphTestee

    /****************************************************************
     * create an exemplary graph to be used in some test methods.
     */
    private DirectedGraphWithDict createGraphTestee(String inputString) 
	throws Exception 
    {
	return testeeCreator.createGraphTestee(inputString) ;
    } // EndMethod createGraphTestee

    /****************************************************************
     * create an exemplary graph within a
     * given context to be used in some test methods.
     */
    private DirectedGraphWithDict createGraphTestee
	(String inputString, DirectedGraphWithDict context) 
	throws Exception 
    {
	return testeeCreator.createGraphTestee(inputString,context) ;
    } // EndMethod createGraphTestee

    // **************************************************************
    public void testEquals() throws Exception {
	// 2 empty graphs should be equals
	{
	    DirectedGraphWithDict empty1 = new DirectedGraphWithDictImpl() ;
	    DirectedGraphWithDict empty2 = new DirectedGraphWithDictImpl() ;
	    assertTrue(empty1.equals(empty2)) ;
	    assertTrue(empty2.equals(empty1)) ;
	}
	// With one node
	{
	    NamedVertex oneNode = new NamedVertex("A") ;
	    DirectedGraphWithDict oneNodeGraph1 = new DirectedGraphWithDictImpl() ;
	    DirectedGraphWithDict oneNodeGraph2 = new DirectedGraphWithDictImpl() ;
	    oneNodeGraph1.add(oneNode) ;
	    assertTrue(!oneNodeGraph1.equals(oneNodeGraph2)) ;
	    assertTrue(!oneNodeGraph2.equals(oneNodeGraph1)) ;
	    oneNodeGraph2.add(oneNode) ;
	    assertTrue( oneNodeGraph1.equals(oneNodeGraph2)) ;
	    assertTrue( oneNodeGraph2.equals(oneNodeGraph1)) ;
	}
	// 2 nodes and one Edge
        {
	    NamedVertex A  = new NamedVertex("A") ;
	    NamedVertex B  = new NamedVertex("B") ;
	    NamedEdge  AB = new NamedEdge(A,B) ;
	    DirectedGraphWithDict twoNodeGraph1 = new DirectedGraphWithDictImpl() ;
	    DirectedGraphWithDict twoNodeGraph2 = new DirectedGraphWithDictImpl() ;
	    twoNodeGraph1.add(A) ;
	    twoNodeGraph1.add(B) ;
	    twoNodeGraph2.add(A) ;
	    twoNodeGraph2.add(B) ;

	    twoNodeGraph1.addEdge(AB) ;
	    assertTrue(!twoNodeGraph1.equals(twoNodeGraph2) ) ;
	    assertTrue(!twoNodeGraph2.equals(twoNodeGraph1) ) ;

	    twoNodeGraph2.addEdge(AB) ;
	    assertTrue( twoNodeGraph1.equals(twoNodeGraph2) ) ;
	    assertTrue( twoNodeGraph2.equals(twoNodeGraph1) ) ;

	    twoNodeGraph1.addEdge(AB) ;
	    assertTrue( twoNodeGraph1.equals(twoNodeGraph2) ) ;
	    assertTrue( twoNodeGraph2.equals(twoNodeGraph1) ) ;

	    twoNodeGraph1.addEdge( new NamedEdge(A, B,"secondLink") ) ;
	    assertTrue(!twoNodeGraph1.equals(twoNodeGraph2) ) ;
	    assertTrue(!twoNodeGraph2.equals(twoNodeGraph1) ) ;

	    twoNodeGraph2.addEdge( new NamedEdge(A, B,"secondLink") ) ;
	    assertTrue( twoNodeGraph1.equals(twoNodeGraph2) ) ;
	    assertTrue( twoNodeGraph2.equals(twoNodeGraph1) ) ;
	} 

	String graph1 =
	    "<project>" +
	    "  <class name=\"C\" >"          +
	    "    <link target=\"D\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "  </class> " +
	    "</project>" ;

	String graph2 =
	    "<project>" +
	    "  <class name=\"C\" >"          +
	    "    <link target=\"D\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "  </class> " +
	    "</project>" ;

	assertEquals(this.createGraphTestee(graph1),this.createGraphTestee(graph2)) ;

	String graph3 =
	    "<project>" +
	    "  <class name=\"C\" >"          +
	    "    <link target=\"D\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	assertTrue(!this.createGraphTestee(graph1).equals(this.createGraphTestee(graph3))) ;

	String graph4 =
	    "<project>" +
	    "  <class name=\"C\" >"          +
	    "    <link target=\"D\" anAttribute=\"Hallo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;
	try {
	    assertTrue(this.createGraphTestee(graph3).equals(this.createGraphTestee(graph4))) ;
	    fail() ;
	} catch(RuntimeException e) {}

	String graph5 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Hi\" >"          +
	    "    <link target=\"D\" anAttribute=\"Hallo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	try {
	    assertTrue(this.createGraphTestee(graph4).equals(this.createGraphTestee(graph5))) ;
	    fail() ;
	} catch(RuntimeException e) {}

	String graph6 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Bye\" >"          +
	    "    <link target=\"D\" anAttribute=\"Hallo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	try {
	    assertTrue(this.createGraphTestee(graph5).equals(this.createGraphTestee(graph6))) ;
	    fail() ;
	} catch(RuntimeException e) {}

	String graph7 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Bye\" >"          +
	    "    <link target=\"D\" anAttribute=\"Ballo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	try {
	    assertTrue(this.createGraphTestee(graph6).equals(this.createGraphTestee(graph7))) ;
	    fail() ;
	} catch(RuntimeException e) {}

    } // EndMethod testEquals

    // **************************************************************
    public void testExtractForwardClosure() 
	throws Exception 
    {

	DirectedGraphWithDict     graphTestee ;
	graphTestee = this.createGraphTestee() ;
	
	NamedVertex A = graphTestee.getNodeWithName("Arbre") ;
	NamedVertex B = graphTestee.getNodeWithName("BArbreCorde") ;
	NamedVertex C = graphTestee.getNodeWithName("CordeArbre") ;
	NamedVertex Co = graphTestee.getNodeWithName("Corde") ;

	DirectedGraphWithDict resultFromExtraction = 
	    graphTestee.extractForwardClosure( B ) ;

	DirectedGraphWithDict expectedGraph = new DirectedGraphWithDictImpl() ;
	expectedGraph.add( B  ) ;
	expectedGraph.add( C  ) ;
	expectedGraph.add( Co ) ;

	expectedGraph.addEdge( graphTestee.getEdge( B , C  ) ) ;
	expectedGraph.addEdge( graphTestee.getEdge( B , Co ) ) ;
	expectedGraph.addEdge( graphTestee.getEdge( C , Co ) ) ;
	expectedGraph.addEdge( graphTestee.getEdge( C , C  ) ) ;
       	expectedGraph.addEdge( graphTestee.getEdge( Co ,C  ) ) ;

	assertEquals( expectedGraph, resultFromExtraction ) ;
	
	// Here we test extraction when the seed (seedGraphStr) 
	// graph is not completely contained in the 
	// context graph (overlappingGraph)

	DirectedGraphWithDict     context ;
	context = this.createGraphTestee() ;

	XMLGraphReader xmlReader
	    = new XMLGraphReader();

	DirectedGraphWithDict overlappingGraph = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( overlappingGraph , 
	      context          , 
	      new ByteArrayInputStream(graph2String.getBytes()) 
	      ) ;

	String seedGraphStr =
	    "<project>" +

	    "  <class name=\"Arbre\">"           +
	    "    <link target = \"BArbreCorde\" />" +
	    "  </class> " +

	    "  <class name=\"BArbreCorde\">"           +
	    "  </class> " +

	    "</project>" ;

	xmlReader
	    = new XMLGraphReader();

	DirectedGraphWithDict seedGraph = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( seedGraph , 
	      context   , 
	      new ByteArrayInputStream(seedGraphStr.getBytes()) 
	      ) ;

	DirectedGraphWithDict resultFromExtractWithOverlap =
	    overlappingGraph.extractForwardClosureFromGraph
	      ( seedGraph ) ;

	String expected =
	    "<project>" +

	    "  <class name=\"Arbre\">"           +
	    "    <link target = \"BArbreCorde\" />" +
	    "    <link target = \"E\" />" +
	    "  </class> " +

	    "  <class name=\"BArbreCorde\">"           +
	    "  </class> " +

	    "  <class name=\"E\">"           +
	    "  </class> " +

	    "</project>" ;

	expectedGraph = this.createGraphTestee(expected) ;

	assertEquals(expectedGraph,resultFromExtractWithOverlap) ;

    } // EndMethod testExtractForwardClosure

    // **************************************************************
    public void testExtractBackwardClosure() 
	throws Exception 
    {
	DirectedGraphWithDict     graphTestee ;
	graphTestee = this.createGraphTestee() ;
	
	NamedVertex A  = graphTestee.getNodeWithName("Arbre") ;
	NamedVertex B  = graphTestee.getNodeWithName("BArbreCorde") ;

	DirectedGraphWithDict resultFromExtraction = 
	    graphTestee.extractBackwardClosure( B ) ;

	DirectedGraphWithDict expectedGraph = new DirectedGraphWithDictImpl() ;
	expectedGraph.add( A  ) ;
	expectedGraph.add( B  ) ;

	expectedGraph.addEdge( graphTestee.getEdge( A , B  ) ) ;

	assertEquals( expectedGraph, resultFromExtraction ) ;
	
	// Here we test extraction when the seed (seedGraphStr) 
	// graph is not completely contained in the 
	// context graph (overlappingGraph)

	DirectedGraphWithDict     context ;
	context = this.createGraphTestee() ;

	XMLGraphReader xmlReader
	    = new XMLGraphReader();

	DirectedGraphWithDict overlappingGraph = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( overlappingGraph, 
	      context         , 
	      new ByteArrayInputStream(graph2String.getBytes())
	      ) ;

	String seedGraphStr =
	    "<project>" +
	    "  <class name=\"E\">"           +
	    "  </class> " +
	    "  <class name=\"BArbreCorde\">"           +
	    "  </class> " +
	    "</project>" ;

	xmlReader
	    = new XMLGraphReader();

	DirectedGraphWithDict seedGraph = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( seedGraph , 
	      context   ,
	      new ByteArrayInputStream(seedGraphStr.getBytes()) 
	      ) ;

	DirectedGraphWithDict resultFromExtractWithOverlap =
	    overlappingGraph.extractBackwardClosureFromGraph
	      ( seedGraph ) ;

	String expected =
	    "<project>" +
	    "  <class name=\"Arbre\">"    +
	    "    <link target = \"E\" />" +
	    "  </class> " +
	    "  <class name=\"BArbreCorde\">" +
	    "  </class> " +
	    "  <class name=\"E\">"           +
	    "  </class> " +
	    "</project>" ;

	expectedGraph = this.createGraphTestee(expected) ;

	assertEquals(expectedGraph,resultFromExtractWithOverlap) ;
    } // EndMethod testExtractBackwardClosure

    // **************************************************************
    public void testExtractForwardClosureFromGraph() throws Exception
    {
	DirectedGraphWithDict     contextGraph ;
	contextGraph = this.createGraphTestee() ;
	
	NamedVertex A  = contextGraph.getNodeWithName("Arbre") ;
	NamedVertex B  = contextGraph.getNodeWithName("BArbreCorde") ;
	NamedVertex C  = contextGraph.getNodeWithName("CordeArbre") ;
	NamedVertex Co = contextGraph.getNodeWithName("Corde") ;
	
	DirectedGraphWithDict result ;

	// We first test that it correctly works with an empty graph.
	{ 
	    DirectedGraphWithDict emptyGraph = new DirectedGraphWithDictImpl() ;
	    // Empty context graph
	    result = emptyGraph.extractForwardClosureFromGraph(emptyGraph) ;
	    assertEquals("Both should be empty.",emptyGraph,result) ;
	    // Context graph which is not empty
	    result = contextGraph.extractForwardClosureFromGraph(emptyGraph) ;
	    assertEquals("Both should be empty.",emptyGraph,result) ;
	} // EndBlock with an empty graph.

	// The same expected result is used for the following tests.

	DirectedGraphWithDict expectedResult 
	    = new DirectedGraphWithDictImpl() ;
	expectedResult.add(B ) ;
	expectedResult.add(C ) ;
	expectedResult.add(Co) ;
	expectedResult.doEdgeCompletionInContext(contextGraph) ;

	// We test that it correctly works with only one node.
	{
	    DirectedGraphWithDict oneNode = new DirectedGraphWithDictImpl() ;
	    oneNode.add(B) ;
	    result = contextGraph.extractForwardClosureFromGraph(oneNode) ;
	    assertEquals(expectedResult,result) ;
	} // EndBlock with only one node.

	// We test that it correctly works with two nodes.
	{
	    DirectedGraphWithDict twoNodes1 = new DirectedGraphWithDictImpl() ;
	    twoNodes1.add(B ) ;
	    twoNodes1.add(C ) ;
	    twoNodes1.doEdgeCompletionInContext(contextGraph) ;
	    result = contextGraph.extractForwardClosureFromGraph(twoNodes1) ;
	    assertEquals(expectedResult,result) ;
	} // EndBlock with two nodes (1)
	{
	    DirectedGraphWithDict twoNodes2 = new DirectedGraphWithDictImpl() ;
	    twoNodes2.add(B ) ;
	    twoNodes2.add(A ) ;
	    twoNodes2.doEdgeCompletionInContext(contextGraph) ;
	    result = contextGraph.extractForwardClosureFromGraph(twoNodes2) ;
	    assertEquals(contextGraph,result) ;
	} // EndBlock with two nodes (2)

    } // EndMethod testExtractForwardClosureFromGraph

    // **************************************************************

    public void testExtracBackwardClosureFromGraph() throws Exception
    {
	DirectedGraphWithDict     contextGraph ;
	contextGraph = this.createGraphTestee() ;
	
	NamedVertex A  = contextGraph.getNodeWithName("Arbre") ;
	NamedVertex B  = contextGraph.getNodeWithName("BArbreCorde") ;
	NamedVertex C  = contextGraph.getNodeWithName("CordeArbre") ;
	NamedVertex Co = contextGraph.getNodeWithName("Corde") ;
	
	DirectedGraphWithDict result ;

	// We first test that it correctly works with an empty graph.
	{ 
	    DirectedGraphWithDict emptyGraph = new DirectedGraphWithDictImpl() ;
	    // Empty context graph
	    result = emptyGraph.extractBackwardClosureFromGraph(emptyGraph) ;
	    assertEquals("Both should be empty.",emptyGraph,result) ;
	    // Context graph which is not empty
	    result = contextGraph.extractBackwardClosureFromGraph(emptyGraph) ;
	    assertEquals("Both should be empty.",emptyGraph,result) ;
	} // EndBlock with an empty graph.

	// The same expected result is used for the following tests.

	DirectedGraphWithDict expectedResult 
	    = new DirectedGraphWithDictImpl() ;
	expectedResult.add(A ) ;
	expectedResult.add(B ) ;
	expectedResult.doEdgeCompletionInContext(contextGraph) ;

	// We test that it correctly works with only one node.
	{
	    DirectedGraphWithDict oneNode = new DirectedGraphWithDictImpl() ;
	    oneNode.add(B) ;
	    result = contextGraph.extractBackwardClosureFromGraph(oneNode) ;
	    assertEquals(expectedResult,result) ;
	} // EndBlock with only one node.

	// We test that it correctly works with two nodes.
	{
	    DirectedGraphWithDict twoNodes1 = new DirectedGraphWithDictImpl() ;
	    twoNodes1.add(B ) ;
	    twoNodes1.add(A ) ;
	    twoNodes1.doEdgeCompletionInContext(contextGraph) ;
	    result = contextGraph.extractBackwardClosureFromGraph(twoNodes1) ;
	    assertEquals(expectedResult,result) ;
	} // EndBlock with two nodes (1)
	{
	    DirectedGraphWithDict twoNodes2 = new DirectedGraphWithDictImpl() ;
	    twoNodes2.add(C ) ;
	    twoNodes2.add(B ) ;
	    twoNodes2.doEdgeCompletionInContext(contextGraph) ;
	    result = contextGraph.extractBackwardClosureFromGraph(twoNodes2) ;
	    assertEquals(contextGraph,result) ;
	} // EndBlock with two nodes (2)

    } // EndMethod testExtracBackwardClosureFromGraph

    // **************************************************************

    String graphForFixBackForw =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"C\" />" +
	    "    <link target = \"D\" />" +
	    "    <link target = \"B\" />" +
	    "  </class> " +

	    "  <class name=\"B\">"           +
	    "    <link target = \"G\" />" +
	    "  </class> " +

	    "  <class name=\"C\"></class> " +

	    "  <class name=\"D\">"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +

	    "  <class name=\"E\">"           +
	    "    <link target = \"D\" />" +
	    "  </class> " +

	    "  <class name=\"F\">"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +

	    "  <class name=\"G\"></class> " +

	    "</project>" ;

    // **************************************************************

    public void testExtracFixBackwardFromGraph() throws Exception {
	DirectedGraphWithDict baseGraph ;
	DirectedGraphWithDict contextGraph ;
	DirectedGraphWithDict result ;
	DirectedGraphWithDict seed ;
	DirectedGraphWithDict expected ;

	baseGraph = new DirectedGraphWithDictImpl() ;

	// *** SEED WITH ONE NODE

	String graphAStr = "<project><class name=\"A\"></class></project>" ;

	contextGraph = this.createGraphTestee(graphForFixBackForw , baseGraph) ;
	seed         = this.createGraphTestee(graphAStr , baseGraph) ;

	result = contextGraph.extractFixBackwardFromGraph(seed,0) ;
	assertEquals(seed,result) ;

	String expectStr = 
	    "<project>"+
	    "  <class name=\"A\"></class>"+
	    "  <class name=\"D\">"        +
	    "    <link target = \"A\" />" +
	    "  </class> "                 +
	    "  <class name=\"F\">"        +
	    "    <link target = \"A\" />" +
	    "  </class>" +
	    "</project>" ;
	result = contextGraph.extractFixBackwardFromGraph(seed,1) ;
	assertEquals("1",this.createGraphTestee(expectStr),result) ;
	
	expectStr = 
	    "<project>"+
	    "  <class name=\"A\">"        +
	    "    <link target = \"D\" />" +
	    "</class>"                    +
	    "  <class name=\"D\">"        +
	    "    <link target = \"A\" />" +
	    "  </class> "                 +
	    "  <class name=\"F\">"        +
	    "    <link target = \"A\" />" +
	    "  </class>" +
	    "  <class name=\"E\">"        +
	    "    <link target = \"D\" />" +
	    "  </class>" +
	    "</project>" ;
	result   = contextGraph.extractFixBackwardFromGraph(seed,2) ;
	expected = this.createGraphTestee(expectStr) ;
	assertEquals(expected,result) ;
	result   = contextGraph.extractFixBackwardFromGraph(seed,3) ;
	assertEquals(expected,result) ;
	    
	// *** SEED WITH 2 NODE

	String graphADStr = 
	    "<project>"+
	    "  <class name=\"A\"></class>" +
	    "  <class name=\"D\"></class>" +
	    "</project>" ;
	seed   = this.createGraphTestee(graphADStr, baseGraph) ;
	result = contextGraph.extractFixBackwardFromGraph(seed,0) ;
	assertEquals(seed,result) ;
	result = contextGraph.extractFixBackwardFromGraph(seed,1) ;
	assertEquals(expected,result) ;
	result = contextGraph.extractFixBackwardFromGraph(seed,2) ;
	assertEquals(expected,result) ;

    } // EndMethod testExtracFixBackwardFromGraph

    // **************************************************************
    public void testextractFixForwardFromGraph() throws Exception {

	DirectedGraphWithDict baseGraph ;
	DirectedGraphWithDict contextGraph ;
	DirectedGraphWithDict result ;
	DirectedGraphWithDict seed ;
	DirectedGraphWithDict expected ;

	baseGraph = new DirectedGraphWithDictImpl() ;

	// *** SEED WITH ONE NODE

	String graphAStr = "<project><class name=\"A\"></class></project>" ;

	contextGraph = this.createGraphTestee(graphForFixBackForw, baseGraph) ;
	seed         = this.createGraphTestee(graphAStr, baseGraph) ;

	result = contextGraph.extractFixForwardFromGraph(seed,0) ;
	assertEquals(seed,result) ;

	String expectStr = 
	    "<project>"+
	    "  <class name=\"A\">"         +
	    "    <link target = \"B\" />"  +
	    "    <link target = \"C\" />"  +
	    "    <link target = \"D\" />"  +
	    "  </class>"                   +
	    "  <class name=\"B\"></class>" +
	    "  <class name=\"C\"></class>" +
	    "  <class name=\"D\"></class>" +
	    "</project>" ;

	result = contextGraph.extractFixForwardFromGraph(seed,1) ;
	assertEquals(this.createGraphTestee(expectStr),result) ;
	
	expectStr = 
	    "<project>"+
	    "  <class name=\"A\">"         +
	    "    <link target = \"B\" />"  +
	    "    <link target = \"C\" />"  +
	    "    <link target = \"D\" />"  +
	    "  </class>"                   +
	    "  <class name=\"B\">"         +
	    "    <link target = \"G\" />"  +
	    "  </class>"                   +
	    "  <class name=\"C\"></class>" +
	    "  <class name=\"G\"></class>" +
	    "  <class name=\"D\">"         +
	    "    <link target = \"A\" />"  +
	    "  </class>" +
	    "</project>"                   ;

	result   = contextGraph.extractFixForwardFromGraph(seed,2) ;
	expected = this.createGraphTestee(expectStr) ;
	assertEquals("2",expected,result) ;
	result   = contextGraph.extractFixForwardFromGraph(seed,3) ;
	assertEquals("3",expected,result) ;
	    
	// *** SEED WITH 2 NODE

	String graphADStr = 
	    "<project>"+
	    "  <class name=\"A\"></class>" +
	    "  <class name=\"D\"></class>" +
	    "</project>";
	seed   = this.createGraphTestee(graphADStr, baseGraph) ;
	result = contextGraph.extractFixForwardFromGraph(seed,0) ;
	assertEquals("2N0",seed,result) ;
	expectStr = 
	    "<project>"+
	    "  <class name=\"A\">"         +
	    "    <link target = \"B\" />"  +
	    "    <link target = \"C\" />"  +
	    "    <link target = \"D\" />"  +
	    "  </class>"                   +
	    "  <class name=\"B\"></class>" +
	    "  <class name=\"C\"></class>" +
	    "  <class name=\"D\">"         +
	    "    <link target = \"A\" />"  +
	    "  </class>"  +
	    "</project>"                  ;

	result = contextGraph.extractFixForwardFromGraph(seed,1) ;
	assertEquals("2N1",this.createGraphTestee(expectStr),result) ;
	result = contextGraph.extractFixForwardFromGraph(seed,2) ;
// 	System.out.println("RESULT:\n"+result+"\n\n") ;
	assertEquals("2N2",expected,result) ;

    } // EndMethod testextractFixForwardFromGraph

    // **************************************************************
    public void testGetNodesWithName() throws Exception {
	DirectedGraphWithDict     graphTestee ;
	graphTestee = this.createGraphTestee() ;

	NamedVertex Arbre       = graphTestee.getNodeWithName("Arbre") ;
	NamedVertex BArbreCorde = graphTestee.getNodeWithName("BArbreCorde") ;
	NamedVertex CordeArbre  = graphTestee.getNodeWithName("CordeArbre") ;
	NamedVertex Corde       = graphTestee.getNodeWithName("Corde") ;

	java.util.SortedSet result ;
	java.util.SortedSet expectedResult ;

	result = graphTestee.getNodeSWithRegExp("") ;
	assertTrue(result.isEmpty()) ;

	result = graphTestee.getNodeSWithRegExp("rbre") ;
	assertTrue(result.isEmpty()) ;

	result = graphTestee.getNodeSWithRegExp("Arbre") ;
	expectedResult = new java.util.TreeSet() ;
	expectedResult.add(Arbre) ;
	assertEquals(expectedResult, result ) ;

	result = graphTestee.getNodeSWithRegExp("*Arbre") ;
	expectedResult.add(CordeArbre) ;
	assertEquals( expectedResult, result ) ;

	result = graphTestee.getNodeSWithRegExp("Corde*") ;
	expectedResult.clear() ;
	expectedResult.add(Corde) ;
	expectedResult.add(CordeArbre) ;
	assertEquals( expectedResult, result ) ;

	result = graphTestee.getNodeSWithRegExp("*Arbre*") ;
	expectedResult.clear() ;
	expectedResult.add(Arbre) ;
	expectedResult.add(CordeArbre) ;
	expectedResult.add(BArbreCorde) ;
	assertEquals( expectedResult, result ) ;

	result = graphTestee.getNodeSWithRegExp("*") ;
	expectedResult.add(Corde) ;
	assertEquals( expectedResult, result ) ;

	// Ill formed expressions should raise an exception

	try {
	    result = graphTestee.getNodeSWithRegExp("**") ;
	    fail("'**' should raise a NotALimitedWilcardExpression");
	} catch (DirectedGraphWithDict.NotALimitedWilcardExpression 
		 wildcardExcep ) {
	} // EndCatch NotALimitedWilcardExpression

	try {
	    result = graphTestee.getNodeSWithRegExp("*x*y") ;
	    fail("'*x*y' should raise a NotALimitedWilcardExpression");
	} catch (DirectedGraphWithDict.NotALimitedWilcardExpression 
		 wildcardExcep ) {
	} // EndCatch NotALimitedWilcardExpression

	try {
	    result = graphTestee.getNodeSWithRegExp("*x*y*") ;
	    fail("'*x*y*' should raise a NotALimitedWilcardExpression");
	} catch (DirectedGraphWithDict.NotALimitedWilcardExpression 
		 wildcardExcep ) {
	} // EndCatch NotALimitedWilcardExpression
    } // EndMethod testGetNodesWithName

    // **************************************************************
    public void testDoEdgeCompletionInContext() 
    throws Exception {

	DirectedGraphWithDict     graphContext ;
	graphContext = this.createGraphTestee() ;

	NamedVertex Arbre       = graphContext.getNodeWithName("Arbre") ;
	NamedVertex BArbreCorde = graphContext.getNodeWithName("BArbreCorde") ;
	NamedVertex CordeArbre  = graphContext.getNodeWithName("CordeArbre") ;
	NamedVertex Corde       = graphContext.getNodeWithName("Corde") ;
	
	DirectedGraphWithDict emptyGraph   = new DirectedGraphWithDictImpl() ;
	emptyGraph.doEdgeCompletionInContext(graphContext);
	assertTrue(emptyGraph.getVerticesCount()==0) ;

	DirectedGraphWithDict ABCoGraph = new DirectedGraphWithDictImpl() ;
	ABCoGraph.add(Arbre) ;
	ABCoGraph.add(BArbreCorde) ;
	ABCoGraph.add(Corde) ;
	ABCoGraph.addEdge( graphContext.getEdge( BArbreCorde, Corde ));
	
	ABCoGraph.doEdgeCompletionInContext(graphContext) ;
	
	DirectedGraphWithDict expectedGraph = new DirectedGraphWithDictImpl() ;
	expectedGraph.add(Arbre) ;
	expectedGraph.add(BArbreCorde) ;
	expectedGraph.add(Corde) ;

	expectedGraph.addEdge( graphContext.getEdge( Arbre      , BArbreCorde));
	expectedGraph.addEdge( graphContext.getEdge( BArbreCorde, Corde      ));

	assertEquals( expectedGraph, ABCoGraph ) ;

	ABCoGraph.doEdgeCompletionInContext(graphContext) ;
	assertEquals( expectedGraph, ABCoGraph ) ;

    } // EndMethod testDoEdgeCompletionInContext

    // **************************************************************
    public void testAddGraph() throws Exception {
	DirectedGraphWithDict testee = new DirectedGraphWithDictImpl() ;
	NamedVertex A   = testee.provideNamedVertexWithName("A") ;
	NamedVertex B   = testee.provideNamedVertexWithName("B") ;
	NamedEdge  A2B = new NamedEdge(A,B) ;
	testee.addEdge(A2B) ;
	
	DirectedGraphWithDict graphC = new DirectedGraphWithDictImpl() ;
	NamedVertex C = graphC.provideNamedVertexWithName("C") ;
	
 	DirectedGraphWithDict BCE = new DirectedGraphWithDictImpl() ;
	BCE.add(B) ;
	BCE.add(C) ;
	NamedVertex E = BCE.provideNamedVertexWithName("E") ;
	NamedEdge  C2B = new NamedEdge(C,B) ;
	NamedEdge  C2E = new NamedEdge(C,E) ;
	NamedEdge  E2B = new NamedEdge(E,B) ;

	BCE.addEdge(C2B) ;
	BCE.addEdge(C2E) ;
	BCE.addEdge(E2B) ;

	DirectedGraphWithDict AB_Expect = new DirectedGraphWithDictImpl() ;
	AB_Expect.add(A) ;
	AB_Expect.add(B) ;
	AB_Expect.addEdge(A2B) ;

	DirectedGraphWithDict ABC_Expect = new DirectedGraphWithDictImpl() ;
	ABC_Expect.add(A) ;
	ABC_Expect.add(B) ;
	ABC_Expect.add(C) ;
	ABC_Expect.addEdge(A2B) ;
	
	DirectedGraphWithDict ABCE_Expect = new DirectedGraphWithDictImpl() ;
	ABCE_Expect.add(A) ;
	ABCE_Expect.add(B) ;
	ABCE_Expect.add(C) ;
	ABCE_Expect.add(E) ;
	ABCE_Expect.addEdge(A2B) ;
	ABCE_Expect.addEdge(C2B) ;
	ABCE_Expect.addEdge(C2E) ;
	ABCE_Expect.addEdge(E2B) ;

	testee.addGraph(new DirectedGraphWithDictImpl()) ;
	assertEquals("1",AB_Expect,testee) ;

	testee.addGraph(graphC) ;
	assertEquals("2",ABC_Expect,testee) ;

	testee.addGraph(BCE) ;
	assertEquals("3",ABCE_Expect,testee) ;

	// Adding 2 graphs which are equals with respect to names
	// but not with respect to object reference don't create
	// duplicates.

	DirectedGraphWithDict  twin1 ;
	DirectedGraphWithDict  twin2 ;
	twin1 = this.createGraphTestee() ;
	twin2 = this.createGraphTestee() ;

	assertEquals("Twin in Names"            ,twin1,twin2) ;
	twin1.addGraph(twin2) ;
	assertEquals("Twin in Names after join.",twin1,twin2) ;

   } // EndMethod testAddGraph

    // **************************************************************
    public void testRemove() throws Exception {
	DirectedGraphWithDict     baseGraph ;
	baseGraph = this.createGraphTestee() ;

	NamedVertex Arbre       = baseGraph.getNodeWithName("Arbre") ;
	NamedVertex BArbreCorde = baseGraph.getNodeWithName("BArbreCorde") ;

	baseGraph.remove(new NamedVertex("aName")) ;

	baseGraph.remove(Arbre) ;
	assertSame( null, baseGraph.getNodeWithName("Arbre") ) ;
	assertTrue( baseGraph.getVerticesCount() == 3) ;
	assertTrue(!baseGraph.cloneVertices().contains(Arbre)) ;
	assertTrue( baseGraph.cloneVertices().contains(BArbreCorde)) ;

	baseGraph.remove(BArbreCorde) ;
	assertTrue( baseGraph.getVerticesCount() == 2) ;
	assertTrue(!baseGraph.cloneVertices().contains(BArbreCorde)) ;
	
	// Pseudo Classes should be treated as links, and
	// be removed as soon as they don't have no parent or
	// no child

	String withPseudoClassString =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"B\" />" +
	    "  </class> " +
	    "  <class name=\"B\" pseudoClass=\"true\" >"           +
	    "    <link target = \"C\" />" +
	    "    <link target = \"D\" />" +
	    "  </class> " +
	    "  <class name=\"C\" >"           +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "  </class> " +
	    "</project>" ;
	
	baseGraph = this.createGraphTestee(withPseudoClassString) ;
	NamedVertex A  = baseGraph.getNodeWithName("A") ;
	
	baseGraph.remove(A) ;

	String expected =
	    "<project>" +
	    "  <class name=\"C\" >"           +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "  </class> " +
	    "</project>" ;

	assertEquals(this.createGraphTestee(expected),baseGraph) ;

	baseGraph = this.createGraphTestee(withPseudoClassString) ;
	NamedVertex C  = baseGraph.getNodeWithName("C") ;
	NamedVertex D  = baseGraph.getNodeWithName("D") ;
	
	baseGraph.remove(C) ;

	expected =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"B\" />" +
	    "  </class> " +
	    "  <class name=\"B\" pseudoClass=\"true\" >"           +
	    "    <link target = \"D\" />" +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "  </class> " +
	    "</project>" ;

// 	System.out.println("\nRESULT:\n"+baseGraph) ;
	assertEquals(this.createGraphTestee(expected),baseGraph) ;

 	baseGraph.remove(D) ;

	expected =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "  </class> " +
	    "</project>" ;

	assertEquals(this.createGraphTestee(expected),baseGraph) ;

   } // EndMethod testRemove

    public void testCleanAroundVertex() throws Exception {
	String testeeStr =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"E\" />" +
	    "    <link target = \"D\" />" +
	    "  </class> " +
	    "  <class name=\"B\" >"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +
	    "  <class name=\"C\" >"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +
	    "  <class name=\"E\">"           +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target = \"C\" />" +
	    "  </class> " +
	    "</project>" ;

	DirectedGraphWithDict testee = 
	    this.createGraphTestee(testeeStr) ;
	DirectedGraphWithDict firstGraph = 
	    this.createGraphTestee(testeeStr) ;

	NamedVertex A  = testee.getNodeWithName("A") ;
	NamedVertex B  = testee.getNodeWithName("B") ;
	NamedVertex D  = testee.getNodeWithName("D") ;
	testee.cleanAroundVertex(B) ;
	assertEquals(firstGraph,testee) ;
	testee.cleanAroundVertex(D) ;
	assertEquals(firstGraph,testee) ;

	String expected =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"D\" />" +
	    "  </class> " +
	    "  <class name=\"C\" >"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target = \"C\" />" +
	    "  </class> " +
	    "</project>" ;

	testee.cleanAroundVertex(A) ; // B and D are removed
	assertEquals(this.createGraphTestee(expected),testee) ;
	testee.remove(A) ; // Trying to reproduce a bug

    } // EndMethod testCleanAroundVertex

    // **************************************************************
    public void testRemoveEdgesWithIterator() throws Exception {
	String graph1String =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link kind=\"a\" target = \"B\" />" +
	    "    <link kind=\"b\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "    <link kind=\"a\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "  </class> " +
	    "</project>" ;

	String expectedAllLinkRemoved =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "  </class> " +
	    "</project>" ;

	String expectedLinkARemoved =
	    "<project>" +

	    "  <class name=\"A\">"           +
	    "    <link kind=\"b\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "  </class> " +
	    "</project>" ;

	String expectedLinkBRemoved =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link kind=\"a\" target = \"B\" />" +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "    <link kind=\"a\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "  </class> " +
	    "</project>" ;

	DirectedGraphWithDict graph1 ;
	graph1 = this.createGraphTestee(graph1String) ;
	graph1.removeEdgesWithIterator(graph1.getEdgeIterator()) ;
	assertEquals( "AllLinkRemoved",
		      this.createGraphTestee(expectedAllLinkRemoved)
		      ,graph1) ;

	graph1 = this.createGraphTestee(graph1String) ;
	graph1.removeEdgesWithIterator(graph1.getEdgeIteratorKind(null)) ;
	assertEquals( "AllLinkRemoved",
		      this.createGraphTestee(expectedAllLinkRemoved)
		      ,graph1) ;

	graph1 = this.createGraphTestee(graph1String) ;
	graph1.removeEdgesWithIterator(graph1.getEdgeIteratorKind("a")) ;
	assertEquals( "LinkARemoved",
		      this.createGraphTestee(expectedLinkARemoved)
		      ,graph1) ;

	graph1 = this.createGraphTestee(graph1String) ;
	graph1.removeEdgesWithIterator(graph1.getEdgeIteratorKind("b")) ;
	assertEquals( "LinkBRemoved",
		      this.createGraphTestee(expectedLinkBRemoved)
		      ,graph1) ;

	// Testing with different graph for iterator and removeEdge

	graph1String =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link kind=\"a\" target = \"B\" />" +
	    "    <link kind=\"b\" target = \"C\" />" +
	    "    <link kind=\"a\" target = \"E\" />" +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "    <link kind=\"a\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "  </class> " +
	    "  <class name=\"E\">"           +
	    "    <link kind=\"a\" target = \"A\" />" +
	    "    <link kind=\"a\" target = \"E\" />" +
	    "  </class> " +
	    "</project>" ;

	String graph2String =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "    <link kind=\"a\" target = \"C\" />" +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "    <link kind=\"a\" target = \"A\" />" +
	    "  </class> " +
	    "</project>" ;
	
	DirectedGraphWithDict context ;
	DirectedGraphWithDict graph2 ;
	XMLGraphReader xmlReader ;

	context = this.createGraphTestee(graph1String) ;

	xmlReader
	    = new XMLGraphReader();

	graph2 = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( graph2  , 
	      context , 
	      new ByteArrayInputStream(graph2String.getBytes())
	      ) ;

	xmlReader
	    = new XMLGraphReader();

	graph1 = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( graph1  , 
	      context , 
	      new ByteArrayInputStream(graph1String.getBytes()) 
	      ) ;

	graph2.removeEdgesWithIterator(graph1.getEdgeIteratorKind("a")) ;
	
	String expectResultRemoveGraph2 =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "  </class> " +
	    "  <class name=\"B\">"           +
	    "  </class> " +
	    "  <class name=\"C\">"           +
	    "    <link kind=\"a\" target = \"A\" />" +
	    "  </class> " +
	    "</project>" ;

	assertEquals( this.createGraphTestee(expectResultRemoveGraph2)
		      ,graph2) ;

    } // EndMethod testRemoveEdgesWithIterator

    // **************************************************************
    public void testGetLinkWithSourceSinkName() throws Exception {
	String inputString = 
	    "<project>" +
	    "  <class name=\"A\" > " +
	    "    <link target = \"B\" name = \"Link1\" />" +
	    "  </class> " + 
	    "  <class name=\"B\"> " +
	    "    <link target = \"B\" name = \"Link2\" />" +
	    "  </class> " + 
	    "  <class name=\"A\" > " +
	    "    <link target = \"B\" name = \"Link1\" />" +
	    "  </class> " + 
	    "</project>" ;
	
	DirectedGraphWithDict testee ;
	testee = this.createGraphTestee(inputString) ;

	NamedVertex a  = testee.getNodeWithName("A") ;
	NamedVertex b  = testee.getNodeWithName("B") ;
	
	NamedEdge link1 = (NamedEdge)testee.getLinkWithSourceSinkName(a,b,"Link1") ;
	NamedEdge link2 = (NamedEdge)testee.getLinkWithSourceSinkName(b,b,"Link2") ;
	assertTrue( link1!=null ) ;
	assertTrue( link2!=null ) ;
	NamedEdge link3 = (NamedEdge)testee.getLinkWithSourceSinkName(a,b,"Link3") ;
	assertNull(link3) ;
	assertEquals( "Link1", link1.getName() ) ;
	assertEquals( "Link2", link2.getName() ) ;

	assertEquals( a, link1.getSource() ) ;
	assertEquals( b, link1.getSink  () ) ;

	assertEquals( b, link2.getSource() ) ;
	assertEquals( b, link2.getSink  () ) ;
	
    } // EndMethod testGetLinkWithSourceSinkName

    /** ****************************************************
     * {@link PropertyBag} allows to set a subset of properties
     * that are considered for equality test, exclusive of any
     * others. This is useful when doing tests in order to
     * focus on the attributes that are relevant for the test.
     */
    public void testAdaptativeEquality() throws Exception {

	// testEquals should still work correctly because
	// it only uses those attributes.
	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (new String[] { "anAttribute", 
			    "anOtherAttribute"
	    }) ;
	testEquals() ;

	// Back to normal.
	PropertyBag.DebugSetPropertiesRelevantForEquality (null) ;
	testEquals() ;

	// Now the real test

	String graph5 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Hi\" >"          +
	    "    <link target=\"D\" anAttribute=\"Hallo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	String graph6 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Bye\" >"          +
	    "    <link target=\"D\" anAttribute=\"Hallo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	String graph7 =
	    "<project>" +
	    "  <class name=\"C\" anOtherAttribute=\"Bye\" >"          +
	    "    <link target=\"D\" anAttribute=\"Ballo\" /> "     +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target=\"C\" /> "     +
	    "  </class> " +
	    "</project>" ;

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (new String[] { "anAttribute", "XXX" }) ;

	assertTrue(this.createGraphTestee(graph5).equals(this.createGraphTestee(graph6))) ;
	try {
	    assertTrue(this.createGraphTestee(graph6).equals(this.createGraphTestee(graph7))) ;
	    fail() ;
	} catch (Exception e) {}

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (new String[] { "anOtherAttribute", "XXX" }) ;

	assertTrue(this.createGraphTestee(graph6).equals(this.createGraphTestee(graph7))) ;
	try {
	    assertTrue(this.createGraphTestee(graph5).equals(this.createGraphTestee(graph6))) ;
	    fail() ;
	} catch (Exception e) {}

	
    } // EndMethod testAdaptativeEquality

    public void testClone() throws Exception {
	String testeeStr =
	    "<project>" +
	    "  <class name=\"A\">"           +
	    "    <link target = \"E\" />" +
	    "    <link target = \"D\" />" +
	    "  </class> " +
	    "  <class name=\"B\" >"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +
	    "  <class name=\"C\" >"           +
	    "    <link target = \"A\" />" +
	    "  </class> " +
	    "  <class name=\"E\">"           +
	    "  </class> " +
	    "  <class name=\"D\">"           +
	    "    <link target = \"C\" />" +
	    "  </class> " +
	    "</project>" ;

	DirectedGraphWithDict testee = 
	    this.createGraphTestee(testeeStr) ;

	DirectedGraphWithDict clone1 = (DirectedGraphWithDict)testee.clone() ;
	DirectedGraphWithDict clone2 = 
	    new DirectedGraphWithDictImpl((DirectedGraphWithDictImpl)testee) ;

	assertTrue(testee.equalsDebug(clone1)) ;
	assertTrue(testee.equalsDebug(clone2)) ;
	assertTrue(clone1.equalsDebug(clone2)) ;

	NamedVertex vertexA = testee.getNodeWithName("A") ;

	testee.remove(vertexA);

	assertTrue("!testee.equals     (clone1)",!testee.equals     (clone1)) ;
	assertTrue("!testee.equals     (clone2)",!testee.equals     (clone2)) ;
	assertTrue(" clone1.equalsDebug(clone2)", clone1.equalsDebug(clone2)) ;
	
	clone1.remove(vertexA);
	assertTrue( testee.equalsDebug(clone1)) ;
	assertTrue(!testee.equals     (clone2)) ;
	assertTrue(!clone1.equals     (clone2)) ;
	
	clone2.remove(vertexA);
	assertTrue(testee.equalsDebug(clone1)) ;
	assertTrue(testee.equalsDebug(clone2)) ;
	assertTrue(clone1.equalsDebug(clone2)) ;

	NamedVertex vertexB = testee.getNodeWithName("B") ;

	clone1.remove(vertexB);
	assertTrue(!testee.equals     (clone1)) ;
	assertTrue( testee.equalsDebug(clone2)) ;
	assertTrue(!clone1.equals     (clone2)) ;

	clone2.remove(vertexB);
	assertTrue(!testee.equals     (clone1)) ;
	assertTrue(!testee.equals     (clone2)) ;
	assertTrue( clone1.equals     (clone2)) ;

	testee.remove(vertexB);
	assertTrue(testee.equalsDebug(clone1)) ;
	assertTrue(testee.equalsDebug(clone2)) ;
	assertTrue(clone1.equalsDebug(clone2)) ;

    } // EndMethod testClone

} // EndClass TestDirectedGraphWithDictImpl
