/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestXMLTraceSet2CallGraphReader.java
 * 
 * Time-stamp: <2003-03-07 20:09:14 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestXMLTraceSet2CallGraphReader.java,v 1.6 2004/10/06 20:27:42 ftaiani Exp $
 * 
 * @version $Name: release-1-3-rc1 $ $Revision: 1.6 $
 * @since Version 0.1.1
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import java.io.ByteArrayInputStream    ;
import java.io.InputStream    ;
import java.util.* ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.* ;

/**
 * In order to better track causability in a call graph, we introduce a new
 * kind of graphs, called "call graph", that are build from a set of call traces,
 * in which call occurrence with different back trace are represented by
 * distinct nodes, even if they are on the same object and same method.
 * @since 0.1.1
 */
public class TestXMLTraceSet2CallGraphReader 
    extends AbstractCallGraphReaderTest {

    private AbstractXMLReader         readerTestee ;
    private CallGraph                 result ;
    private String                    inputString ;

    public TestXMLTraceSet2CallGraphReader(String name) {
	super(name) ;
    } // EndConstructor TestXMLTraceSet2CallGraphReader

    /**
     * Can be overwritten to test subclasses of AbstractXMLReader
     * that should bevahe as {@link XMLTraceSet2CallGraphReader}
     * within its definition domain.
     */
    protected AbstractXMLReader createGraphReaderTestee() {
	return new XMLTraceSet2CallGraphReader() ;
    }

    /** *********************************************/
    private void testEquality(String aTraceSetString, String aGraphString) throws Exception {

 	readerTestee = createGraphReaderTestee() ;

 	CallGraph result ;
 	result = 
	    (CallGraph)readerTestee
	    .readGraphFromXML(new ByteArrayInputStream(aTraceSetString.getBytes())) ;

	XMLGraphReader graphReader = new XMLGraphReader();

	DirectedGraphWithDict expectedGraph ;
	expectedGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(aGraphString.getBytes()) ) ;

//      	System.out.println("\nEXPECTED\n\n" + expectedGraph ) ;
//      	System.out.println("\nRESULT  \n\n" + result ) ;

	assertTrue( expectedGraph.equalsDebug(result) ) ;
    } // EndMethod testEquality

    /** *********************************************/
    public void testCallGraphConstruction() throws Exception {

//   	System.out.println("\n\n*** TESTCALLGRAPHCONSTRUCTION ***\n\n") ;
 	readerTestee = createGraphReaderTestee() ;

 	CallGraph result ;
 	result = 
	    (CallGraph)readerTestee.readGraphFromXML(getTraceSetXMLStream1()) ;

//   	System.out.println("\nEXPECTED\n\n" + getExpectedGraphForStream1() ) ;
//    	System.out.println("\nRESULT  \n\n" + result ) ;

	assertTrue( getExpectedGraphForStream1().equalsDebug( result )) ;
	
    } // testCallGraphConstruction

    /** *********************************************/
    public void testCallGraphConstructionWithThread() throws Exception {
//    	System.out.println("\n\n*** TESTCALLGRAPHCONSTRUCTIONWITHTHREAD ***\n\n") ;
 	String inputString = 
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"D\" method=\"callOnD\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\"> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\"> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	testEquality(inputString,aCallGraphString);
   }

    public static final String aCallGraphString = 
	"<project type=\"callGraph\" >" +
	"  <class name=\"A::callOnA't1-0\" entity=\"A\" " +
	"         method=\"callOnA\" eventOrder=\"0\" threadID=\"1\" > " +
	"    <link target = \"B::callOnB't1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::callOnB't1-1\" entity=\"B\" " +
	"         method=\"callOnB\" eventOrder=\"1\" threadID=\"1\" kind=\"call\" > " +
	"  </class> " + 
	"  <class name=\"A::callOnA't2-2\" entity=\"A\" " +
	"         method=\"callOnA\" eventOrder=\"2\" threadID=\"2\" > " +
	"    <link target = \"B::callOnB't2-3\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::callOnB't2-3\" entity=\"B\" " +
	"         method=\"callOnB\" eventOrder=\"3\" threadID=\"2\" kind=\"call\" > " +
	"    <link target = \"C::callOnC't2-10\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"D::callOnD't1-4\" entity=\"D\" " +
	"         method=\"callOnD\" eventOrder=\"4\" threadID=\"1\" > " +
	"    <link target = \"B::callOnB't1-5\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::callOnB't1-5\" entity=\"B\" " +
	"         method=\"callOnB\" eventOrder=\"5\" threadID=\"1\"  kind=\"call\" > " +
	"    <link target = \"C::callOnC't1-6\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"C::callOnC't1-6\" entity=\"C\" " +
	"         method=\"callOnC\" eventOrder=\"6\" threadID=\"1\" kind=\"call\" > " +
	"  </class> " + 
	"  <class name=\"A::callOnA't1-7\" entity=\"A\" " +
	"         method=\"callOnA\" eventOrder=\"7\" threadID=\"1\" > " +
	"    <link target = \"B::callOnB't1-8\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::callOnB't1-8\" entity=\"B\" " +
	"         method=\"callOnB\" eventOrder=\"8\" threadID=\"1\" kind=\"call\" > " +
	"    <link target = \"C::callOnC't1-9\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"C::callOnC't1-9\" entity=\"C\" " +
	"         method=\"callOnC\"  eventOrder=\"9\" threadID=\"1\" kind=\"call\" > " +
	"  </class> " + 
	"  <class name=\"C::callOnC't2-10\" entity=\"C\" " +
	"         method=\"callOnC\" eventOrder=\"10\" threadID=\"2\" kind=\"call\" > " +
	"  </class> " + 
	"</project>" ;



    /** ****************************************************
     * When successive traces are indentical,
     * the 2 traces denotes 2 different invocations of the call point
     * that terminate their respective (indentical) stacks. Generally,
     * identical call points in identical successive "stack prefixes"
     * are considered the same invocation (and thus don't lead
     * to the creation of a new "invocation node"). In the 
     * case of terminate call points, we are sure of the
     * disctinction, so we should create distinct invocation nodes.
     */
    public void testCallGraphConstructionWithRepetingStackEnds() throws Exception {

// 	System.out.println("** TESTCALLGRAPHCONSTRUCTIONWITHREPETINGSTACKENDS") ;

	String inputString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	String expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'0\" entity=\"A\" method=\"\" eventOrder=\"0\" > " +
	    "    <link target = \"B::'1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'1\" entity=\"B\" method=\"\" eventOrder=\"1\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::'2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "    <link target = \"C::'3\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::'2\" entity=\"C\" method=\"\" eventOrder=\"2\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "  <class name=\"C::'3\" entity=\"C\" method=\"\" eventOrder=\"3\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult);

	inputString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'0\" entity=\"A\" method=\"\" eventOrder=\"0\" > " +
	    "    <link target = \"B::'1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "    <link target = \"B::'3\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'1\" entity=\"B\" method=\"\" eventOrder=\"1\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::'2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::'2\" entity=\"C\" method=\"\" eventOrder=\"2\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "  <class name=\"B::'3\" entity=\"B\" method=\"\" eventOrder=\"3\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult);

	inputString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'0\" entity=\"A\" method=\"\" eventOrder=\"0\" > " +
	    "    <link target = \"B::'1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'1\" entity=\"B\" method=\"\" eventOrder=\"1\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::'2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::'2\" entity=\"C\" method=\"\" eventOrder=\"2\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult);

    } // EndMethod testCallGraphConstructionWithRepetingStackEnds

    /** *********************************************/
    public void testCallGraphConstructionWithRecursion() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	String expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::callOnA'0\" entity=\"A\" method=\"callOnA\" eventOrder=\"0\" > " +
	    "    <link target = \"B::callOnB'1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::callOnB'1\" entity=\"B\" method=\"callOnB\" eventOrder=\"1\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"C::callOnC'2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::callOnC'2\" entity=\"C\" method=\"callOnC\" eventOrder=\"2\" "+
	    "         kind=\"call\" > " +
	    "    <link target = \"A::callOnA'3\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"A::callOnA'3\" entity=\"A\" method=\"callOnA\" eventOrder=\"3\" "+
	    "         kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult);

    } // EndMethod testCallGraphConstructionWithRecursion

    public static final String aCallGraphString2 = 
	"<project type=\"callGraph\" >" +
	"  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" method=\"\" >"+
	"    <link target = \"B::'t1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	"    <link target = \"C::'t1-2\" kind=\"controlflow\" subkind=\"call\" />" +
	"    <link target = \"G::'t1-6\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::'t1-1\" entity=\"B\" eventOrder=\"1\" threadID=\"1\" "+
	"         kind=\"call\" method=\"\" > " +
	"  </class> " + 
	"  <class name=\"C::'t1-2\" entity=\"C\" eventOrder=\"2\" threadID=\"1\" "+
	"         kind=\"call\" method=\"\" > " +
	"    <link target = \"D::'t2-3\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	"  </class> " + 
	"  <class name=\"D::'t2-3\" entity=\"D\" eventOrder=\"3\" threadID=\"2\" "+
	"         kind=\"threadCreate\" method=\"\" > " +
	"    <link target = \"B::'t2-4\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"B::'t2-4\" entity=\"B\" eventOrder=\"4\" threadID=\"2\" "+
	"         kind=\"call\" method=\"\" > " +
	"    <link target = \"F::'t2-5\" kind=\"controlflow\" subkind=\"call\" />" +
	"  </class> " + 
	"  <class name=\"F::'t2-5\" entity=\"F\" eventOrder=\"5\" threadID=\"2\" "+
	"         kind=\"call\" method=\"\" > " +
	"  </class> " + 
	"  <class name=\"G::'t1-6\" entity=\"G\" eventOrder=\"6\" threadID=\"1\" "+
	"         kind=\"call\" method=\"\" > " +
	"  </class> " + 
	"</project>" ;

    /** *********************************************/
    public void testCallGraphConstructionWithThreadCreation() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"G\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "</traceSet>" ; 

	testEquality(inputString,aCallGraphString2) ;

	// The creation of a new thread should not be referenced
	// within a trace and should raise an exception.

	inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "    <new thread=\"2\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"G\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "</traceSet>" ; 

 	readerTestee = createGraphReaderTestee() ;
	try {
	    readerTestee.readGraphFromXML
		(new ByteArrayInputStream(inputString.getBytes())) ;
	    fail() ;
	} catch (Exception e) {}

    } // EndMethod testCallGraphConstructionWithThreadCreation

    /** *********************************************/
    public void testCallGraphConstructionWithThreadTermination1() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
            "  <end thread=\"1\" />" +
	    "</traceSet>" ; 

        PropertyBag.DebugSetPropertiesRelevantForEquality
            (new String[] {"name","target","subkind","kind"}) ;

        String expectedGraph = 
            "<project type=\"callGraph\" >" +
            "  <class name=\"A::'t1-0\" >"+
            "    <link target = \"B::'t1-1\" kind=\"controlflow\" subkind=\"call\" />" +
            "    <link target = \"A::&lt;death>'t1-2\" "+
            "         kind=\"controlflow\" subkind=\"threadEnd\" />" +
            "  </class> " + 
            "  <class name=\"B::'t1-1\" kind=\"call\"      ></class> " + 
            "  <class name=\"A::&lt;death>'t1-2\" kind=\"threadEnd\" ></class> " + 
           "</project>" ;

	testEquality(inputString,expectedGraph) ;

    } // EndMethod testCallGraphConstructionWithThreadTermination1

    /** *********************************************/
    public void testCallGraphConstructionWithThreadTermination2() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
            "  <end thread=\"2\" />" +
	    "</traceSet>" ; 

        PropertyBag.DebugSetPropertiesRelevantForEquality
            (new String[] {"name","target","subkind","kind"}) ;

        String expectedGraph = 
            "<project type=\"callGraph\" >" +
            "  <class name=\"A::'t1-0\" >"+
            "    <link target = \"B::'t1-1\" kind=\"controlflow\" subkind=\"call\" />" +
            "  </class> " + 
            "  <class name=\"B::'t1-1\" kind=\"call\" ></class> " + 
           "</project>" ;

	testEquality(inputString,expectedGraph) ;

    } // EndMethod testCallGraphConstructionWithThreadTermination2

    /** *********************************************/
    public void testCallGraphConstructionWithThreadTermination3() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
            "  <new thread=\"2\" />" +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"C\" method=\"\" />" + 
	    "    <call entity=\"B\" method=\"\" />" + 
            "  </trace> " +
            "  <end thread=\"2\" />" +
            "  <end thread=\"1\" />" +
	    "</traceSet>" ; 

        PropertyBag.DebugSetPropertiesRelevantForEquality
            (new String[] {"name","target","subkind","kind"}) ;

        String expectedGraph = 
            "<project type=\"callGraph\" >" +
            "  <class name=\"A::'t1-0\" >"+
            "    <link target = \"B::'t2-1\" kind=\"controlflow\" subkind=\"threadCreate\"/>" +
            "    <link target = \"A::&lt;death>'t1-4\" kind=\"controlflow\" subkind=\"threadEnd\" />" +
            "  </class> " + 
            "  <class name=\"B::'t2-1\" kind=\"threadCreate\" >"+
            "    <link target = \"C::'t2-2\" kind=\"controlflow\" subkind=\"call\" />" +
            "    <link target = \"B::&lt;death>'t2-3\" kind=\"controlflow\" "+
            "          subkind=\"threadEnd\" />" +
            "  </class> " + 
            "  <class name=\"C::'t2-2\" kind=\"call\" >"+
            "  </class> " + 
            "  <class name=\"B::&lt;death>'t2-3\" kind=\"threadEnd\" ></class> " + 
            "  <class name=\"A::&lt;death>'t1-4\" kind=\"threadEnd\" ></class> " + 
           "</project>" ;

	testEquality(inputString,expectedGraph) ;

    } // EndMethod testCallGraphConstructionWithThreadTermination3

    /**
     * (Francois Taiani) Fri Nov 29 2002 10:09:22
     * I've detected a subtle bug that is activated when the thread creation
     * occurs after an empty trace. The reader gets confused and destroys the
     * set of root event, and becomes unable to recognize events that have
     * already been recorded.
     * This test case reproduces the bug activation.
     */
    public void testCallGraphConstructionWithConfusingThreadCreation() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"0\" > " +
	    "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"G\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "</traceSet>" ; 

	testEquality(inputString,aCallGraphString2) ;

    } // EndMethod testCallGraphConstructionWithConfusingThreadCreation

    /** ****************************************************
     * In some traces, the thread creation as registered by gdb
     * is delayed. Because by default the graph builder consider
     * the top of the last stack trace as being the point of
     * thread creation, wrong thread creation links are created.
     * <pre>
     *   &lt;trace thread="2" > 
     *     &lt;call entity="" method="clone" this="" />
     *     &lt;call entity="" method="pthread_handle_create" this="" />
     *     &lt;call entity="" method="__pthread_manager" this="" />
     *     &lt;call entity="" method="pthread_start_thread" this="" />
     *   &lt;/trace>
     *   &lt;trace thread="4" > 
     *     &lt;call entity="" method="gettimeofday" this="" />
     *     &lt;call entity="" method="__pthread_timedsuspend_new" this="" />
     *     &lt;call entity="" method="pthread_cond_timedwait_relative" this="" />
     *     &lt;call entity="" method="pthread_cond_timedwait" this="" />
     *     &lt;call entity="" method="pthread_start_thread_event" this="" />
     *   &lt;/trace>
     *   &lt;new thread="5" />
     * </pre>
     * For instance in the previous trace extract, the graph builder
     * considers that thread 5 was created within gettimeofday, which 
     * is wrong.<p>
     * In order to avoid that, it's possible to add an attribute
     * to the <traceSet> tag that indicate likely thread creation points.
     * (here "clone"). The last encountered likeley thread creation point
     * is considered to find the point where the thread was created.
     */
    public void testCallGraphConstructionWithDelayedThreadCreation() throws Exception {

 	String inputString = 
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"clone\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "    <call entity=\"C\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ;

	String expectedGraph = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" "+
	    "         method=\"\" >"+
	    "    <link target = \"B::clone't1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::clone't1-1\" entity=\"B\" eventOrder=\"1\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"clone\" > " +
	    "  </class> " + 
	    "  <class name=\"C::'t2-2\" entity=\"C\" eventOrder=\"2\" threadID=\"2\" "+
	    "         method=\"\" > " +
	    "    <link target = \"D::'t2-3\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"D::'t2-3\" entity=\"D\" eventOrder=\"3\" threadID=\"2\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "    <link target = \"E::'t3-4\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	    "  </class> " + 
	    "  <class name=\"E::'t3-4\" entity=\"E\" eventOrder=\"4\" threadID=\"3\" "+
	    "         kind=\"threadCreate\" method=\"\" > " +
	    "  </class> " + 
	    "</project>" ;

// 	System.out.println(XMLTraceSet2CallGraphReader.ReadCallGraphFromString(inputString)) ;
	testEquality(inputString,expectedGraph) ;

	inputString = 
	    "<traceSet threadCreation=\"clone\" >" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"clone\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "    <call entity=\"C\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ;

	testEquality(inputString,expectedGraph) ;

	inputString = 
	    "<traceSet threadCreation=\"B::clone\" >" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"clone\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "    <call entity=\"C\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ;

	expectedGraph = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" "+
	    "         method=\"\" >"+
	    "    <link target = \"B::clone't1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::clone't1-1\" entity=\"B\" eventOrder=\"1\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"clone\" > " +
	    "    <link target = \"E::'t3-4\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	    "  </class> " + 
	    "  <class name=\"C::'t2-2\" entity=\"C\" eventOrder=\"2\" threadID=\"2\" "+
	    "         method=\"\" > " +
	    "    <link target = \"D::'t2-3\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"D::'t2-3\" entity=\"D\" eventOrder=\"3\" threadID=\"2\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "  <class name=\"E::'t3-4\" entity=\"E\" eventOrder=\"4\" threadID=\"3\" "+
	    "         kind=\"threadCreate\" method=\"\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedGraph) ;

    } // EndMethod testCallGraphConstructionWithDelayedThreadCreation

    /** *********************************************/
    public void testCallGraphConstructionWithObjectConstruction() throws Exception {

	String inputString =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"B\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;  

	String expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'0\" entity=\"A\" method=\"\" eventOrder=\"0\" > " +
	    "    <link target = \"B::B'1\" kind=\"controlflow\" subkind=\"create\" />" +
	    "  </class> " + 
	    "  <class name=\"B::B'1\" entity=\"B\" method=\"B\" eventOrder=\"1\" "+
	    "         kind=\"create\" > " +
	    "  </class> " + 
	    "</project>" ;
	
	testEquality(inputString,expectedResult) ;

	inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"B\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;  

	expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" method=\"\" eventOrder=\"0\" threadID=\"1\" > " +
	    "    <link target = \"B::B't1-1\" kind=\"controlflow\" subkind=\"create\" />" +
	    "  </class> " + 
	    "  <class name=\"B::B't1-1\" entity=\"B\" method=\"B\" eventOrder=\"1\" "+
	    "         threadID=\"1\" kind=\"create\" > " +
	    "  </class> " + 
	    "</project>" ;
	
	testEquality(inputString,expectedResult) ;

	inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"A::B\" method=\"B\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;  

	expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" method=\"\" eventOrder=\"0\" threadID=\"1\" > " +
	    "    <link target = \"A::B::B't1-1\" kind=\"controlflow\" subkind=\"create\" />" +
	    "  </class> " + 
	    "  <class name=\"A::B::B't1-1\" entity=\"A::B\" method=\"B\" eventOrder=\"1\" "+
	    "         threadID=\"1\" kind=\"create\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult) ;

	inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"A::B_1\" method=\"B_1\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;  

	expectedResult = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" method=\"\" eventOrder=\"0\" threadID=\"1\" > " +
	    "    <link target = \"A::B_1::B_1't1-1\" kind=\"controlflow\" subkind=\"create\" />" +
	    "  </class> " + 
	    "  <class name=\"A::B_1::B_1't1-1\" entity=\"A::B_1\" method=\"B_1\" eventOrder=\"1\" "+
	    "         threadID=\"1\" kind=\"create\" > " +
	    "  </class> " + 
	    "</project>" ;

	testEquality(inputString,expectedResult) ;

    } // EndMethod testCallGraphConstructionWithObjectConstruction

   /** *********************************************/
    public void testCallGraphConstructionWithCyclingBehavior() throws Exception {

	String traceStringWithCycles = 
	    "<traceSet>" +
	    "    <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"B\" method=\"methodB1\" this=\"\" />" +
	    "    <call entity=\"A\" method=\"methodA\" this=\"\" />" +
	    "  </trace>" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"B\" method=\"methodB2\" this=\"\" />" +
	    "    <call entity=\"A\" method=\"methodA\" this=\"\" />" +
	    "  </trace>" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"B\" method=\"methodB1\" this=\"\" />" +
	    "    <call entity=\"A\" method=\"methodA\" this=\"\" />" +
	    "  </trace>" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"B\" method=\"methodB2\" this=\"\" />" +
	    "    <call entity=\"A\" method=\"methodA\" this=\"\" />" +
	    "  </trace>" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"B\" method=\"methodB1\" this=\"\" />" +
	    "    <call entity=\"A\" method=\"methodA\" this=\"\" />" +
	    "  </trace>" +
	    "</traceSet>" ;

 	CallGraph result ;
 	readerTestee = createGraphReaderTestee() ;
 	result = 
	    (CallGraph)readerTestee
	    .readGraphFromXML(new ByteArrayInputStream(traceStringWithCycles.getBytes())) ;
	Set B1EventS = result.getNodeSWithRegExp("B::methodB1'*");
	assertEquals("B1EventS.size()",3,B1EventS.size())  ;
	Set B2EventS = result.getNodeSWithRegExp("B::methodB2'*");
	assertEquals("B2EventS.size()",2,B2EventS.size())  ;
	Set AEventS = result.getNodeSWithRegExp("A::methodA'*");
	assertEquals("AEventS.size()",1,AEventS.size())  ;

    } // Method testCallGraphConstructionWithCyclingBehavior

} // EndClass TestXMLTraceSet2CallGraphReader
