/*
 * Copyright 2005 Francois Taiani (f.taiani@computer.org)
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestSomeOptions.java
 * 
 * Time-stamp: <>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: TestSomeOptions.java,v 1.2 2005/10/27 12:18:14 ftaiani Exp $
 * 
 */

/**
 * @author Francois Taiani   <f.taiani@computer.org>
 */

package laas.friends.cosmopen.test;

import laas.friends.cosmopen.* ;
import laas.friends.cosmopen.utils.*;
import laas.friends.cosmopen.interpreter.*;

import junit.framework.TestSuite ;

import java.io.* ;

public class TestSomeOptions  
extends junit.framework.TestCase {

    public TestSomeOptions(String name) {
	super(name) ;
    } // EndConstructor TestSomeOptions

    private static String myMiniGraph =
	    "<project>" +
	    "  <class name=\"Node1\">"           +
	    "    <link target = \"Node2\" />" +
	    "  </class> " +
	    "</project>" ;


    private OPSBrowserInterpreter testee ;
    private File miniGraphXMLFile ;
    private File miniGraphDotFile ;

    public void testLeftRightToBottomSwitch() throws Exception {

	testee.interpretCommand("load "      + miniGraphXMLFile) ;

	testee.interpretCommand("2dot base " + miniGraphDotFile);
	String dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(!string1ContainsString2(dotResult," rankdir = \"LR\"")) ;

	testee.interpretCommand("set leftright");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(string1ContainsString2(dotResult, " rankdir = \"LR\"")) ;
	assertTrue(!string1ContainsString2(dotResult," rankdir = \"TB\"")) ;

	testee.interpretCommand("unset leftright");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(!string1ContainsString2(dotResult," rankdir = \"LR\"")) ;

	//	testee.interpretCommand("set leftright");
	
    } // EndMethod testLeftRightToBottomSwitch

    public void testNoPaginationSwitch() throws Exception {

	testee.interpretCommand("load "      + miniGraphXMLFile) ;

	testee.interpretCommand("2dot base " + miniGraphDotFile);
	String dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(string1ContainsString2(dotResult," page = \""));
	assertTrue(string1ContainsString2(dotResult," size = \"")) ;

	testee.interpretCommand("set nopage");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(!string1ContainsString2(dotResult," page = \""));
	assertTrue(!string1ContainsString2(dotResult," size = \"")) ;

	testee.interpretCommand("unset nopage");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue(string1ContainsString2(dotResult," page = \""));
	assertTrue(string1ContainsString2(dotResult," size = \"")) ;

    }

    public void testLandscapeSwitch() throws Exception {

	testee.interpretCommand("load "      + miniGraphXMLFile) ;

	testee.interpretCommand("2dot base " + miniGraphDotFile);
	String dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue
	    (!string1ContainsString2(dotResult,"orientation = \"landscape\"")) ;
	

	testee.interpretCommand("set landscape");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue
	    (string1ContainsString2(dotResult,"orientation = \"landscape\"")) ;
	assertTrue
	    (!string1ContainsString2(dotResult,"orientation = \"porrtrait\"")) ;

	testee.interpretCommand("unset landscape");
	testee.interpretCommand("2dot base " + miniGraphDotFile);
	dotResult = readAndFormatFile(miniGraphDotFile);
	assertTrue
	    (!string1ContainsString2(dotResult,"orientation = \"landscape\"")) ;
    }

    /** ****************************************************
     * {@link CommandInterpreter#out} is redirected to
     * a {@link laas.friends.cosmopen.utils.NullOutStream}.
     */
    protected void setUp() throws java.lang.Exception {

	CommandInterpreter.out = new PrintStream(new NullOutStream()) ;
	
	miniGraphXMLFile = File.createTempFile("opsbrowser", "tmp.xml") ;
	PrintWriter miniGraphXMLWriter
		= new PrintWriter(new FileOutputStream(miniGraphXMLFile)) ;
	miniGraphXMLWriter.print(myMiniGraph);
	miniGraphXMLWriter.close();

	// We create an empty temp file to get a valid temporary name

	miniGraphDotFile = File.createTempFile("opsbrowser", "tmp.dot") ;
		
	testee = new OPSBrowserInterpreter();

    } // EndMethod setUp

    protected void tearDown() throws java.lang.Exception {
	miniGraphDotFile.delete() ;
	miniGraphXMLFile.delete() ;

        CommandInterpreter.out = System.out ;
    }

    private final static boolean string1ContainsString2(String s1, String s2) {
	return (s1.indexOf(s2)!=-1) ;
    } // EndMethod string1ContainsString2

//     private final static String removeSpaceAfter ="=[{(]});:?-<>" ;
//     private final static String removeSpaceBefore="=[{(]});:?-<>" ;
    
//     private final static boolean shouldRemoveSpaceAfter(char someChar) {
// 	return (removeSpaceAfter.indexOf(someChar)!=-1);
//     } // EndMethod shouldRemoveSpaceAfter

//     private final static boolean shouldRemoveSpaceBefore(char someChar) {
// 	return (removeSpaceBefore.indexOf(someChar)!=-1);
//     } // EndMethod shouldRemoveSpaceBefore

    private final static String spaceAround ="=[{(]});:?-<>" ;
    
    private final static boolean shouldPlaceSpaceAround(char someChar) {
	return (spaceAround.indexOf(someChar)!=-1);
    } // EndMethod shouldRemoveSpaceAfter


    /**
     * reads the content of a file into a string and formats it. This
     * methods removed spaces around "=" charachters, removes
     * redundant spaces, newlines characters, spaces after opening
     * parenthesis, before closing parenthesis, before punctuation
     * marks.An alternative would be to use regular expression, but
     * there are not supported by JDK 1.3
     */
    private String readAndFormatFile(File inputFile) throws Exception {

	InputStreamReader reader = new FileReader(inputFile) ;
	StringBuffer result = new StringBuffer() ;
	boolean whiteSpaceBefore = false ;
	boolean previousWasSlash = false ;
	boolean inEmbeddedString = false ;
	boolean closingQuote = false ;
	int nextCharAsInt ;

	while((nextCharAsInt=reader.read())!=-1) {

	    char nextChar = (char)nextCharAsInt ;

// 	    System.err.print(nextChar) ;

	    // If we are at the end of an embedded string, switch off
	    // embedded string mode

	    closingQuote = 
		inEmbeddedString && 
		(nextChar=='"')  && 
		!previousWasSlash ;

	    if (closingQuote)
		inEmbeddedString = false ;

	    // If in embedded string mode, just append characters with
	    // not treatment. Otherwise format string.

	    if (inEmbeddedString) {
		result.append(nextChar) ;
	    }
	    else {

		// If the current character is a whitespace we don't
		// append it to the result buffer immediately. We just
		// flag it with whiteSpaceBefore. This way we can
		// merge contiguous white spaces into one blank
		// characters.

		if (Character.isWhitespace(nextChar)) {
		    whiteSpaceBefore = true ;
		    continue ;
		}
	    
		// The next character is not a space.

		if (shouldPlaceSpaceAround(nextChar) || whiteSpaceBefore)
		    result.append(' ') ;

		result.append(nextChar);

		// If the new non-white character requires white space
		// around it we "virtually" insert one by flagging
		// whiteSpaceBefore. The actual insertion will takes
		// place when a new non-white character is encountered.

		whiteSpaceBefore = shouldPlaceSpaceAround(nextChar) ;

		if ((nextChar=='"') && !closingQuote && !previousWasSlash)
		    inEmbeddedString = true ;

	    } // EndElse !inEmbeddedString

	    previousWasSlash = (nextChar=='\\') ;

	} 
	reader.close() ;
// 	System.err.println(result);
	return result.toString() ;
    } // EndMethod readAndFormatFile

} // EndClass TestSomeOptions
