package salvo.jesus.util;

import java.util.*;

public class Collections {

    /**
     * Returns a subset of the specified collection whose elements
     * match a given object using a specified Comparator. Only
     * elements in the collection that match the given object
     * compareTo using the specified comparator are returned in the
     * subset.
     *
     * @param c          Collection object from which a subset will be taken.
     * @param compareTo  Object to which elements of the collection
     *                   will be compared against.
     * @param comparator Comparator object determining how elements in
     *                   the collection will be compared to the compareTo object.
     * @return Collection object that is a subset of the collection
     *         specified in the argument.
     */
    public static Set subset( Collection c, Object compareTo, Comparator comparator ) {
	Set       subset = new HashSet();
	Iterator  iterator = c.iterator();
	Object    currentobject;

	while( iterator.hasNext() ) {
	    currentobject = iterator.next();
	    if( comparator.compare( compareTo, currentobject ) == 0 )
		c.add( currentobject );
	}

	return subset;
    }

    /**
     * Returns true if the specified Collection has an object that matches
     * the argument object compareTo using the given Comparator. This therefore
     * does use obj1.equals( obj2 ) but uses comparator.compare( obj1, obj2 ).
     *
     * @param c   Collection object from which a subset will be taken.
     * @param compareTo   Object to which elements of the collection will be compared against.
     * @param comparator  Comparator object determining how elements in the collection will
     *                      be compared to the compareTo object.
     * @return boolean   True if the Collection has a matching object
     */
    public static Object contains( Collection c, Object compareTo, Comparator comparator ) {
	Set       subset = new HashSet();
	Iterator  iterator = c.iterator();
	Object    currentobject;

	while( iterator.hasNext() ) {
	    currentobject = iterator.next();
	    if( comparator.compare( compareTo, currentobject ) == 0 )
		return currentobject;
	}

	return null;
    }

    public static Map twoLevelCopyOfMapOfCollectionS(Map mapOfCollectionSToBeCopied) {

	Map result = new HashMap(mapOfCollectionSToBeCopied) ; 

	// LOOP OVER ALL MAP ENTRIES OF THE NEW COPY

	// In each entry when clone the value, which is, in this case,
	// a Collection

	for(Iterator i = result.entrySet().iterator() ; 
	    i.hasNext() ; ) 

	    {
		Map.Entry  entry = (Map.Entry)i.next() ;
		Collection oldValue = (Collection)entry.getValue();
		entry.setValue(new HashSet(oldValue)) ;
		
	    } // EndFor

	return result ;

    } // EndMethod twoLevelCopyOfMapOfCollection

    public static Collection twoLevelCopyOfCollectionOfCollectionS
	(Collection collOfCollSToBeCopied) {
	
	Set result = new HashSet((int)(collOfCollSToBeCopied.size()/0.75*1.20)) ;

	for(Iterator i = collOfCollSToBeCopied.iterator() ;
	    i.hasNext() ; ) 

	    {
		Collection oldCollection = (Collection)i.next() ;
		result.add(new HashSet(oldCollection)) ;
		
	    } // EndFor
	return result ;
    }

} // EndClass Collections
