/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestMonitorGDB.cpp
 * 
 * Francois Taiani   <taiani@laas.fr>
 * Time-stamp: <2003-01-29 20:20:12 ftaiani>
 * 
 */
#include "TestMonitorGDB.h"

#include <string.h>

#include <cppunit/SourceLine.h>
#include <cppunit/TestAssert.h>

/****************************************************/
bool TestMonitorGDB::genericTestBreakPDetectionOnSymbolAndOutput
              (const char * symbol, 
	       const char * output )
{

  strncpy(lastReadLineS, output, LENGTHOFBUFFER);

    
  char * regexForBreakPDetection 
    = createRegularExpForBreakpointDectection(symbol) ;

  bool result 
    = lastReadLineSDoesContainRegexp(regexForBreakPDetection) ;

  free(regexForBreakPDetection) ;

  return result ;
  
} // EndMethod genericTestBreakPDetectionOnSymbolAndOutput

/****************************************************/
void TestMonitorGDB::testContinueUntillBreakpointOnFunction () {

  CPPUNIT_ASSERT_MESSAGE
    ( "Could not find breakpoint in library.",
      genericTestBreakPDetectionOnSymbolAndOutput
      ( "accept",
	"Breakpoint 3, 0x40c141c0 in accept () from /lib/libc.so.6" ) 
      );
					      
  CPPUNIT_ASSERT_MESSAGE
    ( "Could not find breakpoint in main source code.",
      genericTestBreakPDetectionOnSymbolAndOutput
      ( "CallStaticMain",
	"Breakpoint 2, CallStaticMain (env=0x8052544, mainClass=0x8052f2c, mainID=0x80cb6c0, mainArgs=0x8052f3c) at java.c:398")
      );

  CPPUNIT_ASSERT_MESSAGE
    ( "Detect breakpoint though non present.",
      !genericTestBreakPDetectionOnSymbolAndOutput
      ( "acceptor",
	"Breakpoint 3, 0x40c141c0 in accept () from /lib/libc.so.6" ) 
      );
					      
} // EndMethod 

/****************************************************/
bool TestMonitorGDB::genericTestIdentifyBreakpointInLastReadLines
              (const char * symbolToIdentify, 
	       const char * readLines )
{

  strncpy(lastReadLineS, readLines, LENGTHOFBUFFER);

  regmatch_t resultOfIdentification ;

  if (!identifyBreakpointInLastReadLines(resultOfIdentification)) 
    return false ;

  char* match = createSubstringFromLastReadLineS( resultOfIdentification.rm_so ,
						  resultOfIdentification.rm_eo ) ;
  bool result 
    = (strcmp(match,symbolToIdentify)==0) ;

  free( match ) ;

  return result ;
  
} // EndMethod genericTestIdentifyBreakpointInLastReadLines

/****************************************************/
void TestMonitorGDB::testIdentifyBreakpointInLastReadLines() {

   CPPUNIT_ASSERT_MESSAGE
     ( "Could not find breakpoint in library.",
       genericTestIdentifyBreakpointInLastReadLines
       ( "accept",
 	"Breakpoint 3, 0x40c141c0 in accept () from /lib/libc.so.6" ) 
       );
					      
   CPPUNIT_ASSERT_MESSAGE
     ( "Could not find breakpoint in main source code.",
       genericTestIdentifyBreakpointInLastReadLines
       ( "CallStaticMain",
 	"Breakpoint 2, CallStaticMain (env=0x8052544, mainClass=0x8052f2c, mainID=0x80cb6c0, mainArgs=0x8052f3c) at java.c:398")
      );

  CPPUNIT_ASSERT_MESSAGE
    ( "Detect breakpoint though non present.",
      !genericTestBreakPDetectionOnSymbolAndOutput
      ( "acceptor",
	"Breakpoint 3, 0x40c141c0 in accept () from /lib/libc.so.6" ) 
      );
					      
} // EndMethod testIdentifyBreakpointInLastReadLines
